@@ -119,6 +119,7 @@ def CloseStoreWindow ():
 	if Inventory:
 		GUIINV.OpenInventoryWindow ()
 	else:
+		GemRB.GamePause (0, 1)
 		GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE) #enabling the game control screen
 		GemRB.UnhideGUI () #enabling the other windows
 		GUICommonWindows.SetSelectionChangeHandler( None )
@@ -147,6 +148,8 @@ def OpenStoreWindow ():
 		Inventory = 1
 	else:
 		Inventory = None
+		# pause the game, so we don't get interrupted
+		GemRB.GamePause (1, 1)
 
 	GemRB.SetVar ("Action", 0)
 	if GUICommon.GameIsIWD2():
@@ -881,7 +884,7 @@ def RedrawStoreIdentifyWindow ():
 			Flags = GemRB.IsValidStoreItem (pc, inventory_slots[TopIndex+i], ITEM_PC)
 			Item = GemRB.GetItem (Slot['ItemResRef'])
 			Button.SetItemIcon (Slot['ItemResRef'], 0)
-			if Item['StackAmount'] > 1:
+			if Item['MaxStackAmount'] > 1:
 				Button.SetText (str(Slot['Usages0']))
 			else:
 				Button.SetText ("")
@@ -988,26 +991,33 @@ def InfoWindow (Slot, Item):
 
 	MessageWindow = Window = GemRB.LoadWindow (12)
 
+	# TODO: check non-bg2 games to see which label is which
+	if GUICommon.GameIsBG2():
+		NameLabel = Window.GetControl (0x10000000)
+		FakeLabel = Window.GetControl (0x10000007)
+	else:
+		NameLabel = Window.GetControl (0x10000007)
+		FakeLabel = Window.GetControl (0x10000000)
+
 	#fake label
-	Label = Window.GetControl (0x10000000)
-	Label.SetText ("")
+	FakeLabel.SetText ("")
 
 	#description bam
 	if GUICommon.GameIsBG1() or GUICommon.GameIsBG2():
 		Button = Window.GetControl (7)
+		Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_CENTER_PICTURES | IE_GUI_BUTTON_NO_IMAGE, OP_OR)
 		Button.SetItemIcon (Slot['ItemResRef'], 2)
 
 	#slot bam
 	Button = Window.GetControl (2)
 	Button.SetItemIcon (Slot['ItemResRef'], 0)
 
-	Label = Window.GetControl (0x10000007)
 	TextArea = Window.GetControl (5)
 	if Identify:
-		Label.SetText (Item['ItemNameIdentified'])
+		NameLabel.SetText (Item['ItemNameIdentified'])
 		TextArea.SetText (Item['ItemDescIdentified'])
 	else:
-		Label.SetText (Item['ItemName'])
+		NameLabel.SetText (Item['ItemName'])
 		TextArea.SetText (Item['ItemDesc'])
 
 	#Done
@@ -1116,7 +1126,7 @@ def SetupItems (pc, Slot, Button, Label, i, type, idx, steal=0):
 
 		Item = GemRB.GetItem (Slot['ItemResRef'])
 		Button.SetItemIcon (Slot['ItemResRef'], 0)
-		if Item['StackAmount']>1:
+		if Item['MaxStackAmount']>1:
 			Button.SetText ( str(Slot['Usages0']) )
 		else:
 			Button.SetText ("")
@@ -1165,24 +1175,31 @@ def SetupItems (pc, Slot, Button, Label, i, type, idx, steal=0):
 				Button.SetState (IE_GUI_BUTTON_DISABLED)
 
 		if Flags & SHOP_ID:
-			GemRB.SetToken ("ITEMNAME", GemRB.GetString (Item['ItemName']))
+			Name = GemRB.GetString (Item['ItemName'])
 			Button.EnableBorder (0, 1)
 			if not steal and type == ITEM_PC:
 				Price = 1
 		else:
-			GemRB.SetToken ("ITEMNAME", GemRB.GetString (Item['ItemNameIdentified']))
+			Name = GemRB.GetString (Item['ItemNameIdentified'])
 			Button.EnableBorder (0, 0)
 
+		GemRB.SetToken ("ITEMNAME", Name)
 		if Inventory:
 			if GUICommon.GameIsIWD1() or GUICommon.GameIsIWD2():
-				Label.SetText (24890)
+				LabelText = GemRB.GetString (24890)
 			elif GUICommon.GameIsBG2():
-				Label.SetText (28337)
+				LabelText = GemRB.GetString (28337)
 			else:
-				Label.SetText ("")
+				LabelText = ""
 		else:
 			GemRB.SetToken ("ITEMCOST", str(Price) )
-			Label.SetText (10162)
+			LabelText = GemRB.GetString(10162)
+		if type == ITEM_STORE:
+			if steal:
+				LabelText = Name
+			elif Slot["Amount"] != -1:
+				LabelText = LabelText + " (" + str(Slot["Amount"]) + ")"
+		Label.SetText (LabelText)
 	return
 
 def GetRealPrice (pc, mode, Item, Slot):
@@ -1191,7 +1208,7 @@ def GetRealPrice (pc, mode, Item, Slot):
 
 	# FIXME: actually, the price for wands depends on their charge
 	# how ...
-	if Item['StackAmount']>1:
+	if Item['MaxStackAmount']>1:
 		price = price * Slot['Usages0']
 
 	# modifier from store properties (in percent)
@@ -1215,12 +1232,12 @@ def GetRealPrice (pc, mode, Item, Slot):
 			if count > 2:
 				count = 2
 			mod -= count * Store['Depreciation']
+	else:
+		# charisma modifier (in percent)
+		mod += GemRB.GetAbilityBonus (IE_CHR, GemRB.GetPlayerStat (BarteringPC, IE_CHR) - 1, 0)
 
-	# charisma modifier (in percent)
-	mod += GemRB.GetAbilityBonus (IE_CHR, GemRB.GetPlayerStat (BarteringPC, IE_CHR) - 1, 0)
-
-	# reputation modifier (in percent, but absolute)
-	mod = mod * RepModTable.GetValue (0, GemRB.GameGetReputation()/10 - 1) / 100
+		# reputation modifier (in percent, but absolute)
+		mod = mod * RepModTable.GetValue (0, GemRB.GameGetReputation()/10 - 1) / 100
 
 	return price * mod / 100
 
@@ -1375,7 +1392,7 @@ def BuyHeal ():
 
 	GemRB.GameSetPartyGold (gold-Cure['Price'])
 	pc = GemRB.GameGetSelectedPCSingle ()
-	GemRB.ApplySpell (pc, Cure['CureResRef'], Store['StoreOwner'])
+	GemRB.ApplySpell (pc, Cure['CureResRef'], pc)
 	UpdateStoreHealWindow ()
 	return
 
