@@ -127,12 +127,14 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 		rb = 9
 
 	# Rest
-	Button = Window.GetControl (rb)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.RestPress)
-	Button.SetTooltip (11942)
+	if Window.HasControl (rb):
+		Button = Window.GetControl (rb)
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.RestPress)
+		Button.SetTooltip (11942)
 
 	if PortraitWindow:
 		UpdatePortraitWindow ()
+	return
 
 def MarkMenuButton (WindowIndex):
 	Pressed = WindowIndex.GetControl( GemRB.GetVar ("SelectedWindow") )
@@ -143,6 +145,25 @@ def MarkMenuButton (WindowIndex):
 
 	if Pressed: # don't draw the selection when returning to the game
 		Pressed.SetState (IE_GUI_BUTTON_SELECTED)
+	return
+
+def OptionsPress ():
+	"""Toggles the options pane """
+	PP = GemRB.GetMessageWindowSize () & GS_OPTIONPANE
+	if PP:
+		GemRB.GameSetScreenFlags (GS_OPTIONPANE, OP_NAND)
+	else:
+		GemRB.GameSetScreenFlags (GS_OPTIONPANE, OP_OR)
+	return
+
+def PortraitPress ():
+	"""Toggles the portraits pane """
+	PP = GemRB.GetMessageWindowSize () & GS_PORTRAITPANE
+	if PP:
+		GemRB.GameSetScreenFlags (GS_PORTRAITPANE, OP_NAND)
+	else:
+		GemRB.GameSetScreenFlags (GS_PORTRAITPANE, OP_OR)
+	return
 
 def AIPress ():
 	"""Toggles the party AI."""
@@ -152,12 +173,15 @@ def AIPress ():
 
 	if AI:
 		GemRB.GameSetScreenFlags (GS_PARTYAI, OP_NAND)
-		Button.SetTooltip (15918)
 		GemRB.SetVar ("AI", 0)
+		Button.SetTooltip (15918)
 	else:
 		GemRB.GameSetScreenFlags (GS_PARTYAI, OP_OR)
-		Button.SetTooltip (15917)
 		GemRB.SetVar ("AI", GS_PARTYAI)
+		Button.SetTooltip (15917)
+
+	#force redrawing, in case a hotkey triggered this function
+	Button.SetVarAssoc ("AI", GS_PARTYAI)
 	return
 
 def EmptyControls ():
@@ -168,7 +192,7 @@ def EmptyControls ():
 		pc = GemRB.GameGetFirstSelectedActor ()
 		#init spell list
 		GemRB.SpellCast (pc, -1, 0, 1)
-		
+
 	GemRB.SetVar ("ActionLevel", 0)
 	Window = ActionsWindow
 	for i in range (12):
@@ -239,6 +263,10 @@ def OpenActionsWindowControls (Window):
 	global ActionsWindow
 
 	ActionsWindow = Window
+	# 1280 and higher don't have this control
+	if not Window.HasControl (62):
+		UpdateActionsWindow ()
+		return
 	# Gears (time) when options pane is down
 	Button = Window.GetControl (62)
 	Label = Button.CreateLabelOnButton (0x1000003e, "NORMAL", 0)
@@ -267,6 +295,9 @@ def SetupItemAbilities(pc, slot):
 	Window = ActionsWindow
 
 	slot_item = GemRB.GetSlotItem(pc, slot, 1)
+	if not slot_item:
+		# empty quickslot
+		return
 	item = GemRB.GetItem (slot_item["ItemResRef"])
 	Tips = item["Tooltips"]
 
@@ -279,7 +310,7 @@ def SetupItemAbilities(pc, slot):
 			Button.SetItemIcon (slot_item['ItemResRef'], i+6)
 			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectItemAbility)
 			Button.SetVarAssoc ("Ability", i)
-	
+
 			Button.SetTooltip ("F%d - %s"%(i+1,GemRB.GetString(Tips[i])) )
 		else:
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
@@ -373,6 +404,38 @@ def ActionQWeapon3Pressed ():
 def ActionQWeapon4Pressed ():
 	ActionQWeaponPressed(3)
 
+def ActionQSpellPressed (which):
+	pc = GemRB.GameGetFirstSelectedActor ()
+
+	GemRB.SpellCast (pc, -2, which, 1)
+	UpdateActionsWindow ()
+	return
+
+def ActionQSpell1Pressed ():
+	ActionQSpellPressed(0)
+
+def ActionQSpell2Pressed ():
+	ActionQSpellPressed(1)
+
+def ActionQSpell3Pressed ():
+	ActionQSpellPressed(2)
+
+def ActionQSpellRightPressed (which):
+	GemRB.SetVar ("QSpell", which)
+	GemRB.SetVar ("TopIndex", 0)
+	GemRB.SetVar ("ActionLevel", 2)
+	UpdateActionsWindow ()
+	return
+
+def ActionQSpell1RightPressed ():
+	ActionQSpellRightPressed(0)
+
+def ActionQSpell2RightPressed ():
+	ActionQSpellRightPressed(1)
+
+def ActionQSpell3RightPressed ():
+	ActionQSpellRightPressed(2)
+
 #no check needed because the button wouldn't be drawn if illegal
 def ActionLeftPressed ():
 	"""Scrolls the actions window left.
@@ -400,9 +463,9 @@ def ActionRightPressed ():
 	#Type is a bitfield if there is no level given
 	#This is to make sure cleric/mages get all spells listed
 	if Type&128:
-		Max = GemRB.GetKnownSpellsCount(pc, Type&127, -1, 1)
+		Max = GemRB.GetKnownSpellsCount(pc, Type&127, -1, 1, 1)
 	else:
-		Max = GemRB.GetMemorizedSpellsCount(pc, Type, -1, 1)
+		Max = GemRB.GetMemorizedSpellsCount(pc, Type, -1, 1, 1)
 	TopIndex += 10
 	if TopIndex > Max - 10:
 		if Max>10:
@@ -413,8 +476,25 @@ def ActionRightPressed ():
 	UpdateActionsWindow ()
 	return
 
+def ActionMeleePressed ():
+	""" switches to the most damaging melee weapon"""
+	#get the party Index
+	pc = GemRB.GameGetFirstSelectedPC ()
+	GemRB.ExecuteString("EquipMostDamagingMelee()", pc)
+	return
+
+def ActionRangePressed ():
+	""" switches to the most damaging ranged weapon"""
+	#get the party Index
+	pc = GemRB.GameGetFirstSelectedPC ()
+	GemRB.ExecuteString("EquipRanged()", pc)
+	return
+
 def ActionBardSongPressed ():
 	"""Toggles the battle song."""
+
+	##FIXME: check if the actor can actually switch to this state
+	#get the global ID
 	pc = GemRB.GameGetFirstSelectedActor ()
 	GemRB.SetModalState (pc, MS_BATTLESONG, 1)
 	GemRB.PlaySound ("act_01")
@@ -423,6 +503,9 @@ def ActionBardSongPressed ():
 
 def ActionSearchPressed ():
 	"""Toggles detect traps."""
+
+	##FIXME: check if the actor can actually switch to this state
+	#get the global ID
 	pc = GemRB.GameGetFirstSelectedActor ()
 	GemRB.SetModalState (pc, MS_DETECTTRAPS, 1)
 	UpdateActionsWindow ()
@@ -452,6 +535,7 @@ def ActionUseItemPressed ():
 
 def ActionCastPressed ():
 	"""Opens the spell choice scrollbar."""
+	GemRB.SetVar ("QSpell", -1)
 	GemRB.SetVar ("TopIndex", 0)
 	GemRB.SetVar ("ActionLevel", 2)
 	UpdateActionsWindow ()
@@ -521,6 +605,7 @@ def ActionQWeapon4RightPressed ():
 
 def ActionInnatePressed ():
 	"""Opens the innate spell scrollbar."""
+	GemRB.SetVar ("QSpell", -1)
 	GemRB.SetVar ("TopIndex", 0)
 	GemRB.SetVar ("ActionLevel", 3)
 	UpdateActionsWindow ()
@@ -534,6 +619,16 @@ def SpellPressed ():
 	GemRB.GameControlSetTargetMode (TARGET_MODE_CAST)
 	Spell = GemRB.GetVar ("Spell")
 	Type = GemRB.GetVar ("Type")
+	slot = GemRB.GetVar ("QSpell")
+	if slot>=0:
+		#setup quickspell slot
+		#if spell has no target, return
+		#otherwise continue with casting
+		Target = GemRB.SetupQuickSpell (pc, slot, Spell, Type, 1)
+		if Target == 5:
+			Type = -1
+			GemRB.GameControlSetTargetMode (TARGET_MODE_NONE)
+
 	if Type==-1:
 		GemRB.SetVar ("ActionLevel", 0)
 		GemRB.SetVar("Type", 0)
@@ -591,8 +686,10 @@ def OpenPortraitWindow (needcontrols):
 	PortraitWindow = Window = GemRB.LoadWindow (1)
 
 	if needcontrols:
-		Button=Window.GetControl (8)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, MinimizePortraits)
+		# 1280 and higher don't have this control
+		if Window.HasControl (8):
+			Button=Window.GetControl (8)
+			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, MinimizePortraits)
 
 		# AI
 		Button = Window.GetControl (6)
@@ -603,7 +700,7 @@ def OpenPortraitWindow (needcontrols):
 		GemRB.SetVar ("AI", GSFlags)
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, AIPress)
 		Button.SetFlags (IE_GUI_BUTTON_CHECKBOX, OP_OR)
-		Button.SetVarAssoc ("AI", 1)
+		Button.SetVarAssoc ("AI", GS_PARTYAI)
 		if GSFlags:
 			Button.SetTooltip (15917)
 		else:
@@ -653,12 +750,6 @@ def UpdatePortraitWindow ():
 		if Inventory and pc != portid+1:
 			pic = None
 
-		if pic and GemRB.GetPlayerStat(portid+1, IE_STATE_ID) & STATE_DEAD:
-			import GUISTORE
-			# dead pcs are hidden in all stores but temples
-			if GUISTORE.StoreWindow and not GUISTORE.StoreHealWindow:
-				pic = None
-
 		if not pic:
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
