@@ -132,13 +132,13 @@ def SetupMageWindow ():
 			Button = Window.GetControl (3 + i)
 			Button.SetBorder (0,0,0,0,0,0,0,0,64,0,1)
 			Button.SetSprites ("SPELFRAM",0,0,0,0,0)
-			Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_OR)
+			Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_PLAYONCE, OP_OR)
 			Button.SetState (IE_GUI_BUTTON_LOCKED)
 
 	# Setup book spells buttons
 	for i in range (24):
 		Button = Window.GetControl (27 + i)
-		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
+		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_PLAYONCE, OP_OR)
 		Button.SetState (IE_GUI_BUTTON_LOCKED)
 
 	UpdateMageWindow ()
@@ -169,7 +169,8 @@ def UpdateMageWindow ():
 	Label.SetText (Name)
 
 	known_cnt = GemRB.GetKnownSpellsCount (pc, type, level)
-	mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level)
+	mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level, False)
+	true_mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level, True)
 	if not BookType:
 		for i in range (12):
 			Button = Window.GetControl (3 + i)
@@ -177,20 +178,23 @@ def UpdateMageWindow ():
 				ms = GemRB.GetMemorizedSpell (pc, type, level, i)
 				Button.SetSpellIcon (ms['SpellResRef'], 0)
 				Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
-				Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_OR)
+				Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_PLAYONCE, OP_OR)
 				if ms['Flags']:
 					Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenMageSpellUnmemorizeWindow)
 				else:
 					Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnMageUnmemorizeSpell)
 				Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, OpenMageSpellInfoWindow)
-				spell = GemRB.GetSpell (ms['SpellResRef'])
-				Button.SetTooltip (spell['SpellName'])
 				MageMemorizedSpellList.append (ms['SpellResRef'])
 				Button.SetVarAssoc ("SpellButton", i)
 				Button.EnableBorder (0, ms['Flags'] == 0)
+				spell = GemRB.GetSpell (ms['SpellResRef'])
+				if not spell:
+					print "Missing memorised spell!", ms['SpellResRef']
+					continue
+				Button.SetTooltip (spell['SpellName'])
 			else:
 				if i < max_mem_cnt:
-					Button.SetFlags (IE_GUI_BUTTON_NORMAL, OP_SET)
+					Button.SetFlags (IE_GUI_BUTTON_NORMAL | IE_GUI_BUTTON_PLAYONCE, OP_SET)
 				else:
 					Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 				Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
@@ -201,7 +205,7 @@ def UpdateMageWindow ():
 		label = Window.GetControl (0x10000040)
 		if known_cnt:
 			# we give sorcerers all charges for all the spells, so some extra math is needed
-			label.SetText (GemRB.GetString(61256) + " " + str(mem_cnt/known_cnt) + "/" + str(max_mem_cnt))
+			label.SetText (GemRB.GetString(61256) + " " + str(true_mem_cnt/known_cnt) + "/" + str(max_mem_cnt))
 		else:
 			label.SetText ("")
 
@@ -212,10 +216,13 @@ def UpdateMageWindow ():
 			Button.SetSpellIcon (ks['SpellResRef'], 0)
 			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnMageMemorizeSpell)
 			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, OpenMageSpellInfoWindow)
-			spell = GemRB.GetSpell (ks['SpellResRef'])
-			Button.SetTooltip (spell['SpellName'])
 			MageKnownSpellList.append (ks['SpellResRef'])
 			Button.SetVarAssoc ("SpellButton", 100 + i)
+			spell = GemRB.GetSpell (ks['SpellResRef'])
+			if not spell:
+				print "Missing known spell!", ms['SpellResRef']
+				continue
+			Button.SetTooltip (spell['SpellName'])
 
 		else:
 			Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
@@ -308,6 +315,12 @@ def OnMageMemorizeSpell ():
 
 	if GemRB.MemorizeSpell (pc, type, level, index):
 		UpdateMageWindow ()
+		GemRB.PlaySound ("GAM_24")
+		Button = MageWindow.GetControl(index + 27)
+		Button.SetAnimation ("FLASH")
+		mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level, False)
+		Button = MageWindow.GetControl(mem_cnt + 2)
+		Button.SetAnimation ("FLASH")
 	return
 
 def CloseMageSpellUnmemorizeWindow ():
@@ -378,6 +391,9 @@ def OnMageUnmemorizeSpell ():
 
 	if GemRB.UnmemorizeSpell (pc, type, level, index):
 		UpdateMageWindow ()
+		GemRB.PlaySound ("GAM_44")
+		Button = MageWindow.GetControl(index + 3)
+		Button.SetAnimation ("FLASH")
 	return
 
 def OnMageRemoveSpell ():
@@ -700,7 +716,7 @@ def BuildSpellList (pc, type, level):
 
 	SpellList = {}
 	dummy = [Spell1,Spell2,Spell3]
-	mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level)
+	mem_cnt = GemRB.GetMemorizedSpellsCount (pc, type, level, False)
 
 	for i in range(mem_cnt):
 		ms = GemRB.GetMemorizedSpell (pc, type, level, i)
