@@ -252,7 +252,7 @@ def SetupItemAbilities(pc, slot):
 			Button.SetItemIcon (slot_item['ItemResRef'], i+6)
 			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectItemAbility)
 			Button.SetVarAssoc ("Ability", i)
-	
+
 			Button.SetTooltip ("F%d - %s"%(i+1,GemRB.GetString(Tips[i])) )
 		else:
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
@@ -343,6 +343,38 @@ def ActionQWeapon3Pressed ():
 def ActionQWeapon4Pressed ():
 	ActionQWeaponPressed(3)
 
+def ActionQSpellPressed (which):
+	pc = GemRB.GameGetFirstSelectedActor ()
+
+	GemRB.SpellCast (pc, -2, which, 1)
+	UpdateActionsWindow ()
+	return
+
+def ActionQSpell1Pressed ():
+	ActionQSpellPressed(0)
+
+def ActionQSpell2Pressed ():
+	ActionQSpellPressed(1)
+
+def ActionQSpell3Pressed ():
+	ActionQSpellPressed(2)
+
+def ActionQSpellRightPressed (which):
+	GemRB.SetVar ("QSpell", which)
+	GemRB.SetVar ("TopIndex", 0)
+	GemRB.SetVar ("ActionLevel", 2)
+	UpdateActionsWindow ()
+	return
+
+def ActionQSpell1RightPressed ():
+	ActionQSpellRightPressed(0)
+
+def ActionQSpell2RightPressed ():
+	ActionQSpellRightPressed(1)
+
+def ActionQSpell3RightPressed ():
+	ActionQSpellRightPressed(2)
+
 #no check needed because the button wouldn't be drawn if illegal
 def ActionLeftPressed ():
 	"""Scrolls the actions window left.
@@ -369,7 +401,11 @@ def ActionRightPressed ():
 	Type = GemRB.GetVar ("Type")
 	#Type is a bitfield if there is no level given
 	#This is to make sure cleric/mages get all spells listed
-	Max = GemRB.GetMemorizedSpellsCount(pc, Type, -1, 1)
+	if Type&128:
+		Max = GemRB.GetKnownSpellsCount(pc, Type&127, -1, 1, 1)
+	else:
+		Max = GemRB.GetMemorizedSpellsCount(pc, Type, -1, 1, 1)
+
 	TopIndex += 10
 	if TopIndex > Max - 10:
 		if Max>10:
@@ -420,6 +456,8 @@ def ActionUseItemPressed ():
 
 def ActionCastPressed ():
 	"""Opens the spell choice scrollbar."""
+
+	GemRB.SetVar ("QSpell", -1)
 	GemRB.SetVar ("TopIndex", 0)
 	GemRB.SetVar ("ActionLevel", 2)
 	UpdateActionsWindow ()
@@ -427,6 +465,7 @@ def ActionCastPressed ():
 
 def ActionQItemPressed (action):
 	"""Uses the given quick item."""
+
 	pc = GemRB.GameGetFirstSelectedActor ()
 	#quick slot
 	GemRB.UseItem (pc, -2, action, -1, 1)
@@ -502,8 +541,25 @@ def SpellPressed ():
 	GemRB.GameControlSetTargetMode (TARGET_MODE_CAST)
 	Spell = GemRB.GetVar ("Spell")
 	Type = GemRB.GetVar ("Type")
+	slot = GemRB.GetVar ("QSpell")
+	if slot>=0:
+		#setup quickspell slot
+		#if spell has no target, return
+		#otherwise continue with casting
+		Target = GemRB.SetupQuickSpell (pc, slot, Spell, Type, 1)
+		if Target == 5:
+			Type = -1
+			GemRB.GameControlSetTargetMode (TARGET_MODE_NONE)
+
+	if Type==-1:
+		GemRB.SetVar ("ActionLevel", 0)
+		GemRB.SetVar("Type", 0)
 	GemRB.SpellCast (pc, Type, Spell, 1)
-	GemRB.SetVar ("ActionLevel", 0)
+	if GemRB.GetVar ("Type")!=-1:
+		GemRB.SetVar ("ActionLevel", 0)
+		#init spell list
+		GemRB.SpellCast (pc, -1, 0, 1)
+	GemRB.SetVar ("TopIndex", 0)
 	UpdateActionsWindow ()
 	return
 
@@ -672,11 +728,11 @@ def UpdatePortraitWindow ():
 		for x in range(2 - (len(effects)/3)):
 			states = "\n" + states
 		states = "\n" + states
-	
+
 		# blank space
  		# TODO: missing, maybe add another string tag to make glyphs 100% transparent?
 		flag = blank = chr(238)
-	
+
 		# these two are missing too
 		## shopping icon
 		#if pc==portid+1:
