@@ -18,7 +18,8 @@
 #
 #character generation (GUICG 0)
 import GemRB
-from CharOverview import *
+from ie_stats import *
+import CharOverview
 
 def OnLoad():
 	#setting the stats so the feat code will work
@@ -29,5 +30,5 @@ def OnLoad():
 	GemRB.SetPlayerStat (MyChar, IE_INT, GemRB.GetVar ("Ability 4"))
 	GemRB.SetPlayerStat (MyChar, IE_WIS, GemRB.GetVar ("Ability 5"))
 	GemRB.SetPlayerStat (MyChar, IE_CHR, GemRB.GetVar ("Ability 6"))
-	UpdateOverview(6)
+	CharOverview.UpdateOverview(6)
 	return
