@@ -17,16 +17,28 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #
 
-
-# GUICommonWindows.py - functions to open common windows in lower part of the screen
+# GUICommonWindows.py - functions to open common
+# windows in lower part of the screen
+###################################################
 
 import GemRB
+import GUICommon
+import CommonTables
 from GUIDefines import *
-from GUICommon import *
 from ie_stats import *
 from ie_modal import *
 from ie_action import *
 
+import GUIINV
+import GUIJRNL
+import GUIMA
+import GUIOPT
+import GUISPL
+import GUIREC
+import LUCommon
+import InventoryCommon
+import LoadScreen
+
 FRAME_PC_SELECTED = 0
 FRAME_PC_TARGET   = 1
 
@@ -35,6 +47,8 @@ OptionsWindow = None
 DraggedPortrait = None
 
 def SetupMenuWindowControls (Window, Gears, ReturnToGame):
+	"""Sets up all of the basic control windows."""
+
 	global OptionsWindow
 
 	OptionsWindow = Window
@@ -45,41 +59,42 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 	Button.SetTooltip (16309)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetVarAssoc ("SelectedWindow", 0)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenSpellBookWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUISPL.OpenSpellBookWindow)
 
 	# Inventory
 	Button = Window.GetControl (5)
 	Button.SetTooltip (16307)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetVarAssoc ("SelectedWindow", 1)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenInventoryWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIINV.OpenInventoryWindow)
 
 	# Journal
 	Button = Window.GetControl (6)
 	Button.SetTooltip (16308)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetVarAssoc ("SelectedWindow", 2)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenJournalWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIJRNL.OpenJournalWindow)
+
 	# Map
 	Button = Window.GetControl (7)
 	Button.SetTooltip (16310)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetVarAssoc ("SelectedWindow", 3)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenMapWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIMA.OpenMapWindow)
 
 	# Records
 	Button = Window.GetControl (8)
 	Button.SetTooltip (16306)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetVarAssoc ("SelectedWindow", 4)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenRecordsWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIREC.OpenRecordsWindow)
 
 	# Options
 	Button = Window.GetControl (9)
 	Button.SetTooltip (16311)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 	Button.SetVarAssoc ("SelectedWindow", 7)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "OpenOptionsWindow")
+	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIOPT.OpenOptionsWindow)
 
 	# Gears
 	Button = Window.GetControl (10)
@@ -91,21 +106,33 @@ def SetupMenuWindowControls (Window, Gears, ReturnToGame):
 		# Select All
 		Button = Window.GetControl (11)
 		Button.SetTooltip (10485)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SelectAllOnPress")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.SelectAllOnPress)
 		# Rest
 		Button = Window.GetControl (12)
 		Button.SetTooltip (11942)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "RestPress")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.RestPress)
 
 		# Character Arbitration
 		Button = Window.GetControl (13)
 		Button.SetTooltip (16312)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "CharacterWindow")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None) #TODO: CharacterWindow
 	return
 
 def AIPress ():
-	print "AIPress"
-	return
+        """Toggles the party AI."""
+
+        Button = PortraitWindow.GetControl (6)
+        AI = GemRB.GetMessageWindowSize () & GS_PARTYAI
+
+        if AI:
+                GemRB.GameSetScreenFlags (GS_PARTYAI, OP_NAND)
+                Button.SetTooltip (15918)
+                GemRB.SetVar ("AI", 0)
+        else:
+                GemRB.GameSetScreenFlags (GS_PARTYAI, OP_OR)
+                Button.SetTooltip (15917)
+                GemRB.SetVar ("AI", GS_PARTYAI)
+        return
 
 def EmptyControls ():
 	global PortraitWindow
@@ -115,10 +142,13 @@ def EmptyControls ():
 		Button = Window.GetControl (i+6)
 		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 		Button.SetPicture ("")
+		Button.SetText ("")
+		Button.SetActionIcon (globals(), -1)
 	return
 
 def SelectFormationPreset ():
-	GemRB.GameSetFormation ( GemRB.GetVar ("Value"), GemRB.GetVar ("Formation") )
+	"""Choose the default formation."""
+	GemRB.GameSetFormation (GemRB.GetVar ("Value"), GemRB.GetVar ("Formation") )
 	GroupControls ()
 	return
 
@@ -132,7 +162,7 @@ def SetupFormation ():
 		Button.SetSprites ("GUIBTBUT",0,0,1,2,3)
 		Button.SetBAM ("FORM%x"%i,0,0,-1)
 		Button.SetVarAssoc ("Value", i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SelectFormationPreset")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectFormationPreset)
 	return
 
 def GroupControls ():
@@ -140,19 +170,19 @@ def GroupControls ():
 
 	Window = PortraitWindow
 	Button = Window.GetControl (6)
-	Button.SetActionIcon (7)
+	Button.SetActionIcon (globals(), 7)
 	Button = Window.GetControl (7)
-	Button.SetActionIcon (15)
+	Button.SetActionIcon (globals(), 15)
 	Button = Window.GetControl (8)
-	Button.SetActionIcon (21)
+	Button.SetActionIcon (globals(), 21)
 	Button = Window.GetControl (9)
-	Button.SetActionIcon (-1)
+	Button.SetActionIcon (globals(), -1)
 	Button = Window.GetControl (10)
-	Button.SetActionIcon (-1)
+	Button.SetActionIcon (globals(), -1)
 	Button = Window.GetControl (11)
-	Button.SetActionIcon (-1)
+	Button.SetActionIcon (globals(), -1)
 	Button = Window.GetControl (12)
-	Button.SetActionIcon (-1)
+	Button.SetActionIcon (globals(), -1)
 	GemRB.SetVar ("Formation", GemRB.GameGetFormation ())
 	for i in range (5):
 		Button = Window.GetControl (13+i)
@@ -162,8 +192,8 @@ def GroupControls ():
 		Button.SetSprites ("GUIBTBUT",0,0,1,2,3)
 		Button.SetBAM ("FORM%x"%idx,0,0,-1)
 		Button.SetVarAssoc ("Formation", i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "SelectFormation")
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, "SetupFormation")
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.SelectFormation)
+		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, SetupFormation)
 		str = GemRB.GetString (4935)
 		Button.SetTooltip ("F%d - %s"%(8+i,str) )
 	return
@@ -178,52 +208,50 @@ def UpdateActionsWindow ():
 	if PortraitWindow == None:
 		return
 
-	pc = 0
 	#do this only when there is no 'otherwindow'
 	if GemRB.GetVar ("OtherWindow") != -1:
 		return
 
-	for i in range (PARTY_SIZE):
-		if GemRB.GameIsPCSelected (i+1):
-			if pc == 0:
-				pc = i+1
-			else:
-				pc = -1
-				break
-
-	if pc == 0:
+	Selected = GemRB.GetSelectedSize()
+	if Selected == 0:
 		EmptyControls ()
 		return
-	if pc == -1:
+	if Selected > 1:
 		GroupControls ()
 		return
 
+	#we are sure there is only one actor selected
+	pc = GemRB.GameGetFirstSelectedActor ()
+
 	level = GemRB.GetVar ("ActionLevel")
 	TopIndex = GemRB.GetVar ("TopIndex")
 	if level == 0:
-		PortraitWindow.SetupControls (pc, 6)
+		PortraitWindow.SetupControls (globals(), pc, 6, 1)
 	elif level == 1:
-		PortraitWindow.SetupEquipmentIcons (pc, TopIndex, 6)
+		PortraitWindow.SetupEquipmentIcons (globals(), pc, TopIndex, 6, 1)
 	elif level == 2: #spells
 		GemRB.SetVar ("Type", 3)
-		PortraitWindow.SetupSpellIcons (pc, 3, TopIndex, 6)
+		PortraitWindow.SetupSpellIcons (globals(), pc, 3, TopIndex, 6, 1)
 	elif level == 3: #innates
 		GemRB.SetVar ("Type", 4)
-		PortraitWindow.SetupSpellIcons (pc, 4, TopIndex, 6)
+		PortraitWindow.SetupSpellIcons (globals(), pc, 4, TopIndex, 6, 1)
 	return
 
 def ActionQWeaponPressed (which):
+	"""Selects the given quickslot weapon if possible."""
 	global PortraitWindow
 
-	pc = GemRB.GameGetFirstSelectedPC ()
+	pc = GemRB.GameGetFirstSelectedActor ()
+	qs = GemRB.GetEquippedQuickSlot (pc, 1, 1)
 
-	if GemRB.GetEquippedQuickSlot (pc,1)==which and GemRB.GameControlGetTargetMode () != TARGET_MODE_ATTACK:
-		GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK)
+	#38 is the magic slot
+	if ((qs==which) or (qs==38)) and GemRB.GameControlGetTargetMode() != TARGET_MODE_ATTACK:
+		GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK, GA_NO_DEAD|GA_NO_SELF|GA_NO_HIDDEN)
 	else:
 		GemRB.GameControlSetTargetMode (TARGET_MODE_NONE)
-		GemRB.SetEquippedQuickSlot (pc, which)
+		GemRB.SetEquippedQuickSlot (pc, which, -1, 1)
 
-	PortraitWindow.SetupControls (pc, 6)
+	PortraitWindow.SetupControls (globals(), pc, 6)
 	UpdateActionsWindow ()
 	return
 
@@ -241,6 +269,10 @@ def ActionQWeapon4Pressed ():
 
 #no check needed because the button wouldn't be drawn if illegal
 def ActionLeftPressed ():
+	"""Scrolls the actions window left.
+
+	Used primarily for spell selection."""
+
 	TopIndex = GemRB.GetVar ("TopIndex")
 	if TopIndex>10:
 		TopIndex -= 10
@@ -250,11 +282,18 @@ def ActionLeftPressed ():
 	UpdateActionsWindow ()
 	return
 
+#no check needed because the button wouldn't be drawn if illegal
 def ActionRightPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
+	"""Scrolls the action window right.
+
+	Used primarily for spell selection."""
+
+	pc = GemRB.GameGetFirstSelectedActor ()
 	TopIndex = GemRB.GetVar ("TopIndex")
 	Type = GemRB.GetVar ("Type")
-	Max = GemRB.GetMemorizedSpellsCount (pc, Type)
+	#Type is a bitfield if there is no level given
+	#This is to make sure cleric/mages get all spells listed
+	Max = GemRB.GetMemorizedSpellsCount(pc, Type, -1, 1)
 	TopIndex += 10
 	if TopIndex > Max - 10:
 		if Max>10:
@@ -267,28 +306,32 @@ def ActionRightPressed ():
 	return
 
 def ActionBardSongPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
-	GemRB.SetModalState (pc, MS_BATTLESONG)
+	"""Toggles the battle song."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetModalState (pc, MS_BATTLESONG, 1)
 	GemRB.PlaySound ("act_01")
 	UpdateActionsWindow ()
 	return
 
 def ActionSearchPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
-	GemRB.SetModalState (pc, MS_DETECTTRAPS)
+	"""Toggles detect traps."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetModalState (pc, MS_DETECTTRAPS, 1)
 	UpdateActionsWindow ()
 	return
 
 def ActionStealthPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
-	GemRB.SetModalState (pc, MS_STEALTH)
+	"""Toggles stealth."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetModalState (pc, MS_STEALTH, 1)
 	GemRB.PlaySound ("act_07")
 	UpdateActionsWindow ()
 	return
 
 def ActionTurnPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
-	GemRB.SetModalState (pc, MS_TURNUNDEAD)
+	"""Toggles turn undead."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	GemRB.SetModalState (pc, MS_TURNUNDEAD, 1)
 	GemRB.PlaySound ("act_06")
 	UpdateActionsWindow ()
 	return
@@ -300,12 +343,39 @@ def ActionUseItemPressed ():
 	return
 
 def ActionCastPressed ():
-	print "CASRRE"
+	"""Opens the spell choice scrollbar."""
 	GemRB.SetVar ("TopIndex", 0)
 	GemRB.SetVar ("ActionLevel", 2)
 	UpdateActionsWindow ()
 	return
 
+def ActionQItemPressed (action):
+	"""Uses the given quick item."""
+	pc = GemRB.GameGetFirstSelectedActor ()
+	#quick slot
+	GemRB.UseItem (pc, -2, action, -1, 1)
+	return
+
+def ActionQItem1Pressed ():
+	ActionQItemPressed (ACT_QSLOT1)
+	return
+
+def ActionQItem2Pressed ():
+	ActionQItemPressed (ACT_QSLOT2)
+	return
+
+def ActionQItem3Pressed ():
+	ActionQItemPressed (ACT_QSLOT3)
+	return
+
+def ActionQItem4Pressed ():
+	ActionQItemPressed (ACT_QSLOT4)
+	return
+
+def ActionQItem5Pressed ():
+	ActionQItemPressed (ACT_QSLOT5)
+	return
+
 def ActionInnatePressed ():
 	GemRB.SetVar ("TopIndex", 0)
 	GemRB.SetVar ("ActionLevel", 3)
@@ -319,23 +389,25 @@ def ActionSkillsPressed ():
 	return
 
 def SpellPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
+	"""Prepares a spell to be cast."""
+
+	pc = GemRB.GameGetFirstSelectedActor ()
 
 	GemRB.GameControlSetTargetMode (TARGET_MODE_CAST)
 	Spell = GemRB.GetVar ("Spell")
 	Type = GemRB.GetVar ("Type")
-	GemRB.SpellCast (pc, Type, Spell)
+	GemRB.SpellCast (pc, Type, Spell, 1)
 	GemRB.SetVar ("ActionLevel", 0)
 	UpdateActionsWindow ()
 	return
 
 def EquipmentPressed ():
-	pc = GemRB.GameGetFirstSelectedPC ()
+	pc = GemRB.GameGetFirstSelectedActor ()
 
 	GemRB.GameControlSetTargetMode (TARGET_MODE_CAST)
 	Item = GemRB.GetVar ("Equipment")
 	#equipment index
-	GemRB.UseItem (pc, -1, Item)
+	GemRB.UseItem (pc, -1, Item, -1, 1)
 	GemRB.SetVar ("ActionLevel", 0)
 	UpdateActionsWindow ()
 	return
@@ -346,8 +418,8 @@ def GetActorRaceTitle (actor):
 		RaceID += GemRB.GetPlayerStat (actor, IE_RACE)<<16
 	else:
 		RaceID = GemRB.GetPlayerStat (actor, IE_RACE)
-	row = RaceTable.FindValue (3, RaceID )
-	RaceTitle = RaceTable.GetValue (row, 2)
+	row = CommonTables.Races.FindValue (3, RaceID )
+	RaceTitle = CommonTables.Races.GetValue (row, 2)
 	return RaceTitle
 
 # NOTE: this function is called with the primary classes
@@ -355,11 +427,11 @@ def GetKitIndex (actor, ClassIndex):
 	Kit = GemRB.GetPlayerStat (actor, IE_KIT)
 
 	KitIndex = -1
-	ClassID = ClassTable.GetValue (ClassIndex, 2)
+	ClassID = CommonTables.Classes.GetValue (ClassIndex, 2)
 	# skip the primary classes
-	for ci in range (10, ClassTable.GetRowCount ()):
-		BaseClass = ClassTable.GetValue (ci, 3)
-		if BaseClass == ClassID and Kit & ClassTable.GetValue (ci, 2):
+	for ci in range (10, CommonTables.Classes.GetRowCount ()):
+		BaseClass = CommonTables.Classes.GetValue (ci, 3)
+		if BaseClass == ClassID and Kit & CommonTables.Classes.GetValue (ci, 2):
 			KitIndex = ci
 
 	if KitIndex == -1:
@@ -375,9 +447,9 @@ def GetActorClassTitle (actor, ClassIndex):
 	#Class = GemRB.GetPlayerStat (actor, IE_CLASS)
 	KitIndex = GetKitIndex (actor, ClassIndex)
 	if KitIndex == 0:
-		ClassTitle = ClassTable.GetValue (ClassIndex, 0)
+		ClassTitle = CommonTables.Classes.GetValue (ClassIndex, 0)
 	else:
-		ClassTitle = ClassTable.GetValue (KitIndex, 0)
+		ClassTitle = CommonTables.Classes.GetValue (KitIndex, 0)
 
 	if ClassTitle == "*":
 		return 0
@@ -396,12 +468,15 @@ def GetActorPaperDoll (actor):
 SelectionChangeHandler = None
 
 def SetSelectionChangeHandler (handler):
+	"""Updates the selection handler."""
+
 	global SelectionChangeHandler
 
 	# Switching from walking to non-walking environment:
 	# set the first selected PC in walking env as a selected
 	# in nonwalking env
-	if (not SelectionChangeHandler) and handler:
+	#if (not SelectionChangeHandler) and handler:
+	if (not SelectionChangeHandler) and handler and (not GUICommon.NextWindowFn):
 		sel = GemRB.GameGetFirstSelectedPC ()
 		if not sel:
 			sel = 1
@@ -411,50 +486,54 @@ def SetSelectionChangeHandler (handler):
 
 	# redraw selection on change main selection | single selection
 	SelectionChanged ()
+	return
 
 def RunSelectionChangeHandler ():
 	if SelectionChangeHandler:
 		SelectionChangeHandler ()
+	return
 
 def OpenPortraitWindow ():
 	global PortraitWindow
 
 	PortraitWindow = Window = GemRB.LoadWindow (1)
 
-	pc = GemRB.GameGetSelectedPCSingle ()
-	Inventory = GemRB.GetVar ("Inventory")
-
 	for i in range (PARTY_SIZE):
 		Button = Window.GetControl (i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, "PortraitButtonOnPress")
-		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, "PortraitButtonOnShiftPress")
-		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, "OnDropItemToPC")
-		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG, "PortraitButtonOnDrag")
-		Button.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, "PortraitButtonOnMouseEnter")
-		Button.SetEvent (IE_GUI_MOUSE_LEAVE_BUTTON, "PortraitButtonOnMouseLeave")
+		Button.SetFont ("STATES2")
+		Button.SetVarAssoc ("PressedPortrait", i+1)
+
+		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitButtonOnPress)
+		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, PortraitButtonOnShiftPress)
+		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, InventoryCommon.OnDropItemToPC)
+		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP_PORTRAIT, OnDropPortraitToPC)
+		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG, PortraitButtonOnDrag)
+		Button.SetEvent (IE_GUI_MOUSE_ENTER_BUTTON, PortraitButtonOnMouseEnter)
+		Button.SetEvent (IE_GUI_MOUSE_LEAVE_BUTTON, PortraitButtonOnMouseLeave)
 
 		Button.SetFlags (IE_GUI_BUTTON_ALIGN_TOP|IE_GUI_BUTTON_ALIGN_LEFT|IE_GUI_BUTTON_PICTURE, OP_SET)
 
 		Button.SetBorder (FRAME_PC_SELECTED, 1, 1, 2, 2, 0, 255, 0, 255)
 		Button.SetBorder (FRAME_PC_TARGET, 3, 3, 4, 4, 255, 255, 0, 255)
-		Button.SetVarAssoc ("PressedPortrait", i)
-		Button.SetFont ('NUMFONT')
 
 	UpdatePortraitWindow ()
 	SelectionChanged ()
 	return Window
 
 def UpdatePortraitWindow ():
+	"""Updates all of the portraits."""
+
 	Window = PortraitWindow
 
 	pc = GemRB.GameGetSelectedPCSingle ()
 	Inventory = GemRB.GetVar ("Inventory")
 
-	for i in range (PARTY_SIZE):
-		Button = Window.GetControl (i)
-		pic = GemRB.GetPlayerPortrait (i+1, 1)
-		if Inventory and pc != i+1:
+	for portid in range (PARTY_SIZE):
+		Button = Window.GetControl (portid)
+		pic = GemRB.GetPlayerPortrait (portid+1, 1)
+		if Inventory and pc != portid+1:
 			pic = None
+
 		if not pic:
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -462,53 +541,100 @@ def UpdatePortraitWindow ():
 			Button.SetTooltip ("")
 			continue
 
-		Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_ALIGN_BOTTOM|IE_GUI_BUTTON_ALIGN_LEFT|IE_GUI_BUTTON_HORIZONTAL|IE_GUI_BUTTON_DRAGGABLE, OP_SET)
+		Button.SetFlags (IE_GUI_BUTTON_PICTURE| \
+				IE_GUI_BUTTON_ALIGN_BOTTOM| \
+				IE_GUI_BUTTON_ALIGN_LEFT| \
+				IE_GUI_BUTTON_HORIZONTAL| \
+				IE_GUI_BUTTON_DRAGGABLE, OP_SET)
+		Button.SetState (IE_GUI_BUTTON_LOCKED)
 		Button.SetPicture (pic, "NOPORTSM")
+		GUICommon.SetupDamageInfo (portid+1, Button)
 
-		hp = GemRB.GetPlayerStat (i+1, IE_HITPOINTS)
-		hp_max = GemRB.GetPlayerStat (i+1, IE_MAXHITPOINTS)
-		Button.SetText ("%d/%d" %(hp, hp_max))
-		Button.SetTooltip (GemRB.GetPlayerName (i+1, 1) + "\n%d/%d" %(hp, hp_max))
+		#add effects on the portrait
+		effects = GemRB.GetPlayerStates (portid+1)
+		states = ""
+		for col in range(len(effects)):
+			states = effects[col:col+1] + states
+			if col % 3 == 2: states = "\n" + states
+		for x in range(3 - (len(effects)/3)):
+			states = "\n" + states
+		states = "\n" + states
 
+		# blank space
+		flag = blank = chr(238)
+
+		# shopping icon
+		if pc==portid+1:
+			if GemRB.GetStore()!=None:
+				flag = chr(155)
+		# talk icon
+		if GemRB.GameGetSelectedPCSingle(1)==portid+1:
+			flag = chr(154)
+
+		if LUCommon.CanLevelUp (portid+1):
+			states = flag+blank+chr(255) + states
+		else:
+			states = flag+blank+blank + states
+		Button.SetText(states)
 	return
 
 def PortraitButtonOnDrag ():
 	global DraggedPortrait
 
 	#they start from 1
-	DraggedPortrait = GemRB.GetVar ("PressedPortrait")+1
+	DraggedPortrait = GemRB.GetVar ("PressedPortrait")
 	GemRB.DragItem (DraggedPortrait, -1, "")
 	return
 
 def PortraitButtonOnPress ():
+	"""Selects the portrait individually."""
+
 	i = GemRB.GetVar ("PressedPortrait")
 
+	if not i:
+		return
+
+	if GemRB.GameControlGetTargetMode() != TARGET_MODE_NONE:
+		GemRB.ActOnPC (i)
+		return
+
 	if (not SelectionChangeHandler):
-		if GemRB.GameIsPCSelected (i+1):
+		if GemRB.GameIsPCSelected (i):
 			GemRB.GameControlSetScreenFlags (SF_CENTERONACTOR, OP_OR)
-		GemRB.GameSelectPC (i + 1, True, SELECT_REPLACE)
+		GemRB.GameSelectPC (i, True, SELECT_REPLACE)
 	else:
-		GemRB.GameSelectPCSingle (i + 1)
+		GemRB.GameSelectPCSingle (i)
 		SelectionChanged ()
 		RunSelectionChangeHandler ()
 	return
 
 def PortraitButtonOnShiftPress ():
-	i = GemRB.GetVar ('PressedPortrait')
+	"""Handles selecting multiple portaits with shift."""
+
+	i = GemRB.GetVar ("PressedPortrait")
+
+	if not i:
+		return
 
 	if (not SelectionChangeHandler):
-		sel = GemRB.GameIsPCSelected (i + 1)
+		sel = GemRB.GameIsPCSelected (i)
 		sel = not sel
-		GemRB.GameSelectPC (i + 1, sel)
+		GemRB.GameSelectPC (i, sel)
 	else:
-		GemRB.GameSelectPCSingle (i + 1)
+		GemRB.GameSelectPCSingle (i)
 		SelectionChanged ()
 		RunSelectionChangeHandler ()
 	return
 
-# Run by Game class when selection was changed
 def SelectionChanged ():
-	# FIXME: hack. If defined, display single selection
+	"""Ran by the Game class when a PC selection is changed."""
+
+	global PortraitWindow
+
+	if not PortraitWindow:
+		return
+
+	GemRB.SetVar ("ActionLevel", 0)
 	if (not SelectionChangeHandler):
 		UpdateActionsWindow ()
 		for i in range (PARTY_SIZE):
@@ -520,6 +646,8 @@ def SelectionChanged ():
 		for i in range (PARTY_SIZE):
 			Button = PortraitWindow.GetControl (i)
 			Button.EnableBorder (FRAME_PC_SELECTED, i + 1 == sel)
+	import CommonWindow
+	CommonWindow.CloseContainerWindow()
 	return
 
 def PortraitButtonOnMouseEnter ():
@@ -527,23 +655,115 @@ def PortraitButtonOnMouseEnter ():
 
 	i = GemRB.GetVar ("PressedPortrait")
 
-	if DraggedPortrait != None:
-		GemRB.DragItem (0, -1, "")
-		#this might not work
-		GemRB.SwapPCs (DraggedPortrait, i+1)
-		DraggedPortrait = None
+	if not i:
+		return
+
+	GemRB.GameControlSetLastActor( i )
+	if GemRB.IsDraggingItem()==2:
+		if DraggedPortrait != None:
+			GemRB.SwapPCs (DraggedPortrait, i)
+			GemRB.SetVar ("PressedPortrait", DraggedPortrait)
+			DraggedPortrait = i
+			GemRB.SetTimedEvent (CheckDragging, 1)
+		else:
+			OnDropPortraitToPC()
 		return
 
 	if GemRB.IsDraggingItem ():
-		Button = PortraitWindow.GetControl (i)
+		Button = PortraitWindow.GetControl (i-1)
 		Button.EnableBorder (FRAME_PC_TARGET, 1)
 	return
 
+def OnDropPortraitToPC ():
+	GemRB.SetVar ("PressedPortrait",0)
+	GemRB.DragItem (0, -1, "")
+	DraggedPortrait = None
+	return
+
+def CheckDragging():
+	"""Contains portrait dragging in case of mouse out-of-range."""
+
+	global DraggedPortrait
+
+	i = GemRB.GetVar ("PressedPortrait")
+	if not i:
+		GemRB.DragItem (0, -1, "")
+
+	if GemRB.IsDraggingItem()!=2:
+		DraggedPortrait = None
+	return
+
 def PortraitButtonOnMouseLeave ():
 	i = GemRB.GetVar ("PressedPortrait")
-	Button = PortraitWindow.GetControl (i)
+	if not i:
+		return
+
+	Button = PortraitWindow.GetControl (i-1)
 	Button.EnableBorder (FRAME_PC_TARGET, 0)
+	GemRB.SetVar ("PressedPortrait", 0)
+	GemRB.SetTimedEvent (CheckDragging, 1)
 	return
 
+def ActionStopPressed ():
+	for i in GemRB.GetSelectedActors():
+		GemRB.ClearActions (i, 1)
+	return
+
+def ActionTalkPressed ():
+	GemRB.GameControlSetTargetMode (TARGET_MODE_TALK,GA_NO_DEAD|GA_NO_ENEMY|GA_NO_HIDDEN)
+
+def ActionAttackPressed ():
+	GemRB.GameControlSetTargetMode (TARGET_MODE_ATTACK,GA_NO_DEAD|GA_NO_SELF|GA_NO_HIDDEN)
+
+def ActionDefendPressed ():
+	GemRB.GameControlSetTargetMode (TARGET_MODE_DEFEND,GA_NO_SELF|GA_NO_ENEMY|GA_NO_HIDDEN)
+
+def ActionThievingPressed ():
+	GemRB.GameControlSetTargetMode (TARGET_MODE_PICK, GA_NO_DEAD|GA_NO_SELF|GA_NO_ENEMY|GA_NO_HIDDEN)
+
+def OpenWaitForDiscWindow ():
+	global DiscWindow
+
+	if DiscWindow:
+		GemRB.HideGUI ()
+		if DiscWindow:
+			DiscWindow.Unload ()
+		GemRB.SetVar ("OtherWindow", -1)
+		# ...LoadWindowPack()
+		EnableAnimatedWindows ()
+		DiscWindow = None
+		GemRB.UnhideGUI ()
+		return
+
+	try:
+		GemRB.HideGUI ()
+	except:
+		pass
+
+	GemRB.LoadWindowPack ("GUIID")
+	DiscWindow = Window = GemRB.LoadWindow (0)
+	GemRB.SetVar ("OtherWindow", Window.ID)
+	label = DiscWindow.GetControl (0)
+
+	disc_num = GemRB.GetVar ("WaitForDisc")
+	#disc_path = GemRB.GetVar ("WaitForDiscPath")
+	disc_path = 'XX:'
+
+	text = GemRB.GetString (31483) + " " + str (disc_num) + " " + GemRB.GetString (31569) + " " + disc_path + "\n" + GemRB.GetString (49152)
+	label.SetText (text)
+	DisableAnimatedWindows ()
+	# 31483 - Please place PS:T disc number
+	# 31568 - Please place the PS:T DVD
+	# 31569 - in drive
+	# 31570 - Wrong disc in drive
+	# 31571 - There is no disc in drive
+	# 31578 - No disc could be found in drive. Please place Disc 1 in drive.
+	# 49152 - To quit the game, press Alt-F4
+
+	try:
+		GemRB.UnhideGUI ()
+	except:
+		DiscWindow.SetVisible (WINDOW_VISIBLE)
+
 def CheckLevelUp(pc):
-	GemRB.SetVar (CheckLevelUp+str(pc), CanLevelUp (pc))
+	GemRB.SetVar ("CheckLevelUp"+str(pc), LUCommon.CanLevelUp (pc))
