@@ -61,13 +61,16 @@ class Window;
  */
 
 class GEM_EXPORT Control {
+protected:
+	/** Focused Control */
+	bool hasFocus;
 public:
 	Control();
 	virtual ~Control();
 	/** Draws the Control on the Output Display */
 	virtual void Draw(unsigned short x, unsigned short y) = 0;
 	/** Sets the Text of the current control */
-	virtual int SetText(const char* string, int pos = 0) = 0;
+	virtual void SetText(const char* string);
 	/** Sets the Tooltip text of the current control */
 	int SetTooltip(const char* string);
 	/** Displays the tooltip text, Worldmap handles this differently */
@@ -97,8 +100,6 @@ public: // Public attributes
 	/** Text to display as a tooltip when the mouse cursor hovers
 	 * for some time over the control */
 	char* Tooltip;
-	/** Focused Control */
-	bool hasFocus;
 	/** If true, control is redrawn during next call to gc->DrawWindows.
 	 * Then it's set back to false. */
 	bool Changed;
@@ -115,6 +116,8 @@ public: //Events
 	Window *GetOwner() const { return Owner; }
 	/** Set the Flags */
 	int SetFlags(int arg_flags, int opcode);
+	virtual void SetFocus(bool focus);
+	bool isFocused();
 	/** Set handler for specified event. Override in child classes */
 	virtual bool SetEvent(int eventType, EventHandler handler) = 0;
 	/** Run specified handler, it may return error code */
@@ -135,6 +138,8 @@ public: //Events
 	/** Mouse Button Up */
 	virtual void OnMouseUp(unsigned short x, unsigned short y,
 		unsigned short Button, unsigned short Mod);
+	/** Mouse wheel scroll */
+	virtual void OnMouseWheelScroll( short x, short y);
 	/** Special Key Press */
 	virtual void OnSpecialKeyPress(unsigned char Key);
 	virtual bool IsPixelTransparent(unsigned short /*x*/, unsigned short /*y*/) {
