@@ -25,17 +25,25 @@
 #include "DialogHandler.h"
 #include "DisplayMessage.h"
 #include "Effect.h"
+#include "Font.h"
 #include "Game.h"
 #include "GameData.h"
+#include "GlobalTimer.h"
 #include "ImageMgr.h"
 #include "Interface.h"
 #include "Item.h"
+#include "KeyMap.h"
+#include "PathFinder.h"
 #include "SaveGameIterator.h"
 #include "ScriptEngine.h"
+#include "TableMgr.h"
+#include "TextArea.h"
 #include "TileMap.h"
 #include "Video.h"
 #include "damages.h"
 #include "GameScript/GSUtils.h"
+#include "GUI/EventMgr.h"
+#include "GUI/Window.h"
 #include "Scriptable/Container.h"
 #include "Scriptable/Door.h"
 #include "Scriptable/InfoPoint.h"
@@ -46,11 +54,6 @@
 #define DEBUG_SHOW_CONTAINERS   0x02
 #define DEBUG_SHOW_DOORS	DEBUG_SHOW_CONTAINERS
 #define DEBUG_SHOW_LIGHTMAP     0x08
-#ifdef TOUCHSCREEN
-#	define SCROLL_BORDER 32
-#else
-#	define SCROLL_BORDER 5
-#endif
 
 static const Color cyan = {
 	0x00, 0xff, 0xff, 0xff
@@ -79,13 +82,10 @@ static const Color gray = {
 	0x80, 0x80, 0x80, 0xff
 };
 
-//Animation* effect;
-
 #define FORMATIONSIZE 10
 typedef Point formation_type[FORMATIONSIZE];
 ieDword formationcount;
 static formation_type *formations=NULL;
-static bool mqs = false;
 static ieResRef TestSpell="SPWI207";
 
 //If one of the actors has tracking on, the gamecontrol needs to display
@@ -98,15 +98,6 @@ void GameControl::SetTracker(Actor *actor, ieDword dist)
 	distance = dist;
 }
 
-//Multiple Quick saves is an experimental GemRB feature.
-//multiple quick saves are kept, their age is determined by the slot
-//number. There is an algorithm which keeps about log2(n) slots alive.
-//The algorithm is implemented in SaveGameIterator
-void GameControl::MultipleQuickSaves(int arg)
-{
-	mqs=arg==1;
-}
-
 GameControl::GameControl(void)
 {
 	if (!formations) {
@@ -131,9 +122,7 @@ GameControl::GameControl(void)
 	lastCursor = IE_CURSOR_NORMAL;
 	moveX = moveY = 0;
 	scrolling = false;
-#ifdef TOUCHSCREEN
-	touched=false;
-#endif
+	touchScrollAreasEnabled = false;
 	numScrollCursor = 0;
 	DebugFlags = 0;
 	AIUpdateCounter = 1;
@@ -142,6 +131,15 @@ GameControl::GameControl(void)
 
 	ResetTargetMode();
 
+	core->GetDictionary()->Lookup("TouchScrollAreas",tmp);
+	if (tmp) {
+		touchScrollAreasEnabled = true;
+		touched = false;
+		scrollAreasWidth = 32;
+	} else {
+		scrollAreasWidth = 5;
+	}
+
 	core->GetDictionary()->Lookup("Center",tmp);
 	if (tmp) {
 		ScreenFlags=SF_ALWAYSCENTER|SF_CENTERONACTOR;
@@ -270,7 +268,8 @@ void GameControl::CreateMovement(Actor *actor, const Point &p)
 
 	actor->AddAction( action );
 	// force action so that we get target recticles immediately
-	actor->ProcessActions(true);
+	// FIXME
+	actor->ProcessActions();
 }
 
 GameControl::~GameControl(void)
@@ -287,19 +286,6 @@ GameControl::~GameControl(void)
 	}
 }
 
-//Autosave was triggered by the GUI
-void GameControl::AutoSave()
-{
-	core->GetSaveGameIterator()->CreateSaveGame(0, false);
-}
-
-//QuickSave was triggered by the GUI
-//mqs is the 'multiple quick saves' flag
-void GameControl::QuickSave()
-{
-	core->GetSaveGameIterator()->CreateSaveGame(1, mqs == 1);
-}
-
 // ArrowSprite cycles
 //  321
 //  4 0
@@ -586,30 +572,24 @@ void GameControl::Draw(unsigned short x, unsigned short y)
 		}
 	}
 
-#ifdef TOUCHSCREEN
-	if (moveY < 0 && scrolling)
-		video->DrawLine(screen.x+4, screen.y+SCROLL_BORDER, screen.w+screen.x-4, screen.y+SCROLL_BORDER, red);
-	else
-		video->DrawLine(screen.x+4, screen.y+SCROLL_BORDER, screen.w+screen.x-4, screen.y+SCROLL_BORDER, gray);
-	if (moveY > 0 && scrolling)
-		video->DrawLine(screen.x+4, screen.h-SCROLL_BORDER, screen.w+screen.x-4, screen.h-SCROLL_BORDER, red);
-	else
-		video->DrawLine(screen.x+4, screen.h-SCROLL_BORDER, screen.w+screen.x-4, screen.h-SCROLL_BORDER, gray);
-	if (moveX < 0 && scrolling)
-		video->DrawLine(screen.x+SCROLL_BORDER, screen.y+4, screen.x+SCROLL_BORDER, screen.h+screen.y-4, red);
-	else
-		video->DrawLine(screen.x+SCROLL_BORDER, screen.y+4, screen.x+SCROLL_BORDER, screen.h+screen.y-4, gray);
-	if (moveX > 0 && scrolling)
-		video->DrawLine(screen.w+screen.x-SCROLL_BORDER, screen.y+4, screen.w+screen.x-SCROLL_BORDER, screen.h-4, red);
-	else
-		video->DrawLine(screen.w+screen.x-SCROLL_BORDER, screen.y+4, screen.w+screen.x-SCROLL_BORDER, screen.h-4, gray);
-#endif
-}
-
-/** inherited from Control, GameControl doesn't need it */
-int GameControl::SetText(const char* /*string*/, int /*pos*/)
-{
-	return 0;
+	if (touchScrollAreasEnabled) {
+		if (moveY < 0 && scrolling)
+			video->DrawLine(screen.x+4, screen.y+scrollAreasWidth, screen.w+screen.x-4, screen.y+scrollAreasWidth, red);
+		else
+			video->DrawLine(screen.x+4, screen.y+scrollAreasWidth, screen.w+screen.x-4, screen.y+scrollAreasWidth, gray);
+		if (moveY > 0 && scrolling)
+			video->DrawLine(screen.x+4, screen.h-scrollAreasWidth, screen.w+screen.x-4, screen.h-scrollAreasWidth, red);
+		else
+			video->DrawLine(screen.x+4, screen.h-scrollAreasWidth, screen.w+screen.x-4, screen.h-scrollAreasWidth, gray);
+		if (moveX < 0 && scrolling)
+			video->DrawLine(screen.x+scrollAreasWidth, screen.y+4, screen.x+scrollAreasWidth, screen.h+screen.y-4, red);
+		else
+			video->DrawLine(screen.x+scrollAreasWidth, screen.y+4, screen.x+scrollAreasWidth, screen.h+screen.y-4, gray);
+		if (moveX > 0 && scrolling)
+			video->DrawLine(screen.w+screen.x-scrollAreasWidth, screen.y+4, screen.w+screen.x-scrollAreasWidth, screen.h-4, red);
+		else
+			video->DrawLine(screen.w+screen.x-scrollAreasWidth, screen.y+4, screen.w+screen.x-scrollAreasWidth, screen.h-4, gray);
+	}
 }
 
 /** Key Press Event */
@@ -880,7 +860,7 @@ void GameControl::OnKeyRelease(unsigned char Key, unsigned short Mod)
 				area->MoveVisibleGroundPiles(p);
 				break;
 			case 'x': // shows coordinates on the map
-				printf( "%s [%d.%d]\n", area->GetScriptName(), p.x, p.y );
+				print( "%s [%d.%d]\n", area->GetScriptName(), p.x, p.y );
 				break;
 			case 'g'://shows loaded areas and other game information
 				game->DebugDump();
@@ -996,54 +976,40 @@ void GameControl::OnKeyRelease(unsigned char Key, unsigned short Mod)
 				break;
 			case '4': //show all traps and infopoints
 				DebugFlags ^= DEBUG_SHOW_INFOPOINTS;
-				printf("Show traps and infopoints %s\n", DebugFlags & DEBUG_SHOW_INFOPOINTS ? "ON" : "OFF");
+				print("Show traps and infopoints %s\n", DebugFlags & DEBUG_SHOW_INFOPOINTS ? "ON" : "OFF");
 				break;
 			case '6': //show the lightmap
 				DebugFlags ^= DEBUG_SHOW_LIGHTMAP;
-				printf("Show lightmap %s\n", DebugFlags & DEBUG_SHOW_LIGHTMAP ? "ON" : "OFF");
+				print("Show lightmap %s\n", DebugFlags & DEBUG_SHOW_LIGHTMAP ? "ON" : "OFF");
 				break;
 			case '7': //toggles fog of war
 				core->FogOfWar ^= FOG_DRAWFOG;
-				printf("Show Fog-Of-War: %s\n", core->FogOfWar & FOG_DRAWFOG ? "ON" : "OFF");
+				print("Show Fog-Of-War: %s\n", core->FogOfWar & FOG_DRAWFOG ? "ON" : "OFF");
 				break;
 			case '8': //show searchmap over area
 				core->FogOfWar ^= FOG_DRAWSEARCHMAP;
-				printf("Show searchmap %s\n", core->FogOfWar & FOG_DRAWSEARCHMAP ? "ON" : "OFF");
+				print("Show searchmap %s\n", core->FogOfWar & FOG_DRAWSEARCHMAP ? "ON" : "OFF");
 				break;
 			default:
-				printf( "KeyRelease:%d - %d\n", Key, Mod );
+				print( "KeyRelease:%d - %d\n", Key, Mod );
 				break;
 		}
 		return; //return from cheatkeys
 	}
 	switch (Key) {
+//FIXME: move these to guiscript
 		case 'h': //hard pause
 			if (DialogueFlags & DF_FREEZE_SCRIPTS) break;
 			//fallthrough
 		case ' ': //soft pause
 			DialogueFlags ^= DF_FREEZE_SCRIPTS;
-	 		if (DialogueFlags&DF_FREEZE_SCRIPTS) {
+			if (DialogueFlags&DF_FREEZE_SCRIPTS) {
 				displaymsg->DisplayConstantString(STR_PAUSED,0xff0000);
 				SetDisplayText(STR_PAUSED, 0); // time 0 = removed instantly on unpause
-		 	} else {
+			} else {
 				displaymsg->DisplayConstantString(STR_UNPAUSED,0xff0000);
 			}
 			break;
-		case 'm':
-			core->GetGUIScriptEngine()->RunFunction("GUIMA","OpenMapWindow");
-			break;
-		case 'j':
-			core->GetGUIScriptEngine()->RunFunction("GUIJRNL","OpenJournalWindow");
-			break;
-		case 'i':
-			core->GetGUIScriptEngine()->RunFunction("GUIINV","OpenInventoryWindow");
-			break;
-		case 'r':
-			core->GetGUIScriptEngine()->RunFunction("GUIREC","OpenRecordsWindow");
-			break;
-		case 'q': //quicksave
-			QuickSave();
-			break;
 		case GEM_ALT: //alt key (shows containers)
 #ifdef ANDROID
 		case 'c': // show containers in ANDROID, GEM_ALT is not possible to use
@@ -1051,6 +1017,7 @@ void GameControl::OnKeyRelease(unsigned char Key, unsigned short Mod)
 			DebugFlags &= ~DEBUG_SHOW_CONTAINERS;
 			break;
 		default:
+			core->GetKeyMap()->ResolveKey(Key,0);
 			break;
 	}
 }
@@ -1061,8 +1028,7 @@ void GameControl::DisplayTooltip() {
 		Map* area = game->GetCurrentArea( );
 		if (area) {
 			Actor *actor = area->GetActorByGlobalID(lastActorID);
-			if (actor && (actor->GetStat(IE_STATE_ID)&STATE_DEAD || actor->GetInternalFlag()&IF_JUSTDIED)) {
-				// checking IF_JUSTDIED is kind of horrid, but seems necessary
+			if (actor && (actor->GetStat(IE_STATE_ID)&STATE_DEAD || actor->GetInternalFlag()&IF_REALLYDIED)) {
 				// no tooltips for dead actors!
 				actor->SetOver( false );
 				lastActorID = 0;
@@ -1229,66 +1195,67 @@ void GameControl::OnMouseOver(unsigned short x, unsigned short y)
 		return;
 	}
 
-#ifdef TOUCHSCREEN
-	int mousescrollspd = core->GetMouseScrollSpeed();
-	Region region;
-	Map* map;
-	Point mapsize;
-	Region viewport = core->GetVideoDriver()->GetViewport();
-	moveX = 0;
-	moveY = 0;
-	// Top scroll area
-	region=Region(XPos, YPos, Width, YPos+SCROLL_BORDER);
-	if (region.PointInside(x, y)) {
-		// Check for end of map area
-		if (viewport.y > 0)
-			moveY = -mousescrollspd;
-	}
-	// Bottom scroll area
-	region=Region(XPos, Height-SCROLL_BORDER, Width, Height);
-	if (region.PointInside(x, y)) {
-		// Check for end of map area
-		map = core->GetGame()->GetCurrentArea();
-		if (map != NULL) {
-			mapsize = map->TMap->GetMapSize();
-			if((viewport.y + viewport.h) < mapsize.y)
-				moveY = mousescrollspd;
-		}
-	}
-	// Left scroll area
-	region=Region(XPos, YPos, XPos+SCROLL_BORDER, Height);
-	if (region.PointInside(x, y)) {
-		// Check for end of map area
-		if(viewport.x > 0)
-			moveX = -mousescrollspd;
-	}
-	// Right scroll area
-	region=Region(Width-SCROLL_BORDER, YPos, Width, Height);
-	if (region.PointInside(x, y)) {
-		// Check for end of map area
-		map = core->GetGame()->GetCurrentArea();
-		if (map != NULL) {
-			mapsize = map->TMap->GetMapSize();
-			if((viewport.x + viewport.w) < mapsize.x)
-				moveX = mousescrollspd;
-		}
-	}
-	if ((moveX != 0 || moveY != 0) && touched) {
-		scrolling = true;
-		return;
-	} else {
+	Video *video = core->GetVideoDriver();
+
+	if (touchScrollAreasEnabled) {
+		int mousescrollspd = core->GetMouseScrollSpeed();
+		Region region;
+		Map* map;
+		Point mapsize;
+		Region viewport = video->GetViewport();
 		moveX = 0;
 		moveY = 0;
-		scrolling = false;
-		Video* video = core->GetVideoDriver();
-		video->SetDragCursor(NULL);
+		// Top scroll area
+		region=Region(XPos, YPos, Width, YPos+scrollAreasWidth);
+		if (region.PointInside(x, y)) {
+			// Check for end of map area
+			if (viewport.y > 0)
+				moveY = -mousescrollspd;
+		}
+		// Bottom scroll area
+		region=Region(XPos, Height-scrollAreasWidth, Width, Height);
+		if (region.PointInside(x, y)) {
+			// Check for end of map area
+			map = core->GetGame()->GetCurrentArea();
+			if (map != NULL) {
+				mapsize = map->TMap->GetMapSize();
+				if((viewport.y + viewport.h) < mapsize.y)
+					moveY = mousescrollspd;
+			}
+		}
+		// Left scroll area
+		region=Region(XPos, YPos, XPos+scrollAreasWidth, Height);
+		if (region.PointInside(x, y)) {
+			// Check for end of map area
+			if(viewport.x > 0)
+				moveX = -mousescrollspd;
+		}
+		// Right scroll area
+		region=Region(Width-scrollAreasWidth, YPos, Width, Height);
+		if (region.PointInside(x, y)) {
+			// Check for end of map area
+			map = core->GetGame()->GetCurrentArea();
+			if (map != NULL) {
+				mapsize = map->TMap->GetMapSize();
+				if((viewport.x + viewport.w) < mapsize.x)
+					moveX = mousescrollspd;
+			}
+		}
+		if ((moveX != 0 || moveY != 0) && touched) {
+			scrolling = true;
+			return;
+		} else {
+			moveX = 0;
+			moveY = 0;
+			scrolling = false;
+			video->SetDragCursor(NULL);
+		}
 	}
-#endif
 
 	lastMouseX = x;
 	lastMouseY = y;
 	Point p( x,y );
-	core->GetVideoDriver()->ConvertToGame( p.x, p.y );
+	video->ConvertToGame( p.x, p.y );
 	if (MouseIsDown && ( !DrawSelectionRect )) {
 		if (( abs( p.x - StartX ) > 5 ) || ( abs( p.y - StartY ) > 5 )) {
 			DrawSelectionRect = true;
@@ -1460,38 +1427,34 @@ void GameControl::OnGlobalMouseMove(unsigned short x, unsigned short y)
 		return;
 	}
 
-#ifndef TOUCHSCREEN
-	int mousescrollspd = core->GetMouseScrollSpeed();
+	if (!touchScrollAreasEnabled) {
+		int mousescrollspd = core->GetMouseScrollSpeed();
 
-	if (x <= SCROLL_BORDER)
-		moveX = -mousescrollspd;
-	else {
-		if (x >= ( core->Width - SCROLL_BORDER ))
-			moveX = mousescrollspd;
-		else
-			moveX = 0;
-	}
-	if (y <= SCROLL_BORDER)
-		moveY = -mousescrollspd;
-	else {
-		if (y >= ( core->Height - SCROLL_BORDER ))
-			moveY = mousescrollspd;
-		else
-			moveY = 0;
-	}
+		if (x <= scrollAreasWidth)
+			moveX = -mousescrollspd;
+		else {
+			if (x >= ( core->Width - scrollAreasWidth ))
+				moveX = mousescrollspd;
+			else
+				moveX = 0;
+		}
+		if (y <= scrollAreasWidth)
+			moveY = -mousescrollspd;
+		else {
+			if (y >= ( core->Height - scrollAreasWidth ))
+				moveY = mousescrollspd;
+			else
+				moveY = 0;
+		}
 
-	if (moveX != 0 || moveY != 0) {
-		scrolling = true;
-	} else if (scrolling) {
-		scrolling = false;
+		if (moveX != 0 || moveY != 0) {
+			scrolling = true;
+		} else if (scrolling) {
+			scrolling = false;
+		}
 
-		Video* video = core->GetVideoDriver();
-		video->SetDragCursor(NULL);
+		core->GetVideoDriver()->SetDragCursor(NULL);
 	}
-#else
-(void)x;
-(void)y;
-#endif
 }
 
 void GameControl::UpdateScrolling() {
@@ -1644,6 +1607,7 @@ void GameControl::TryToCast(Actor *source, const Point &tgt)
 	} else {
 		action->int0Parameter = spellSlot;
 		action->int1Parameter = spellIndex;
+		action->int2Parameter = UI_SILENT;
 	}
 	source->AddAction( action );
 	if (!spellCount) {
@@ -1691,6 +1655,7 @@ void GameControl::TryToCast(Actor *source, Actor *tgt)
 	} else {
 		action->int0Parameter = spellSlot;
 		action->int1Parameter = spellIndex;
+		action->int2Parameter = UI_SILENT;
 	}
 	source->AddAction( action );
 	if (!spellCount) {
@@ -1737,7 +1702,7 @@ void GameControl::HandleContainer(Container *container, Actor *actor)
 		return;
 	}
 
-	if ((target_mode == TARGET_MODE_PICK)) {
+	if (target_mode == TARGET_MODE_PICK) {
 		TryToPick(actor, container);
 		return;
 	}
@@ -1797,14 +1762,16 @@ bool GameControl::HandleActiveRegion(InfoPoint *trap, Actor * actor, Point &p)
 		//don't bother with this region further
 		return true;
 	}
-	if ((target_mode == TARGET_MODE_PICK)) {
+	if (target_mode == TARGET_MODE_PICK) {
 		TryToDisarm(actor, trap);
 		return true;
 	}
 
 	switch(trap->Type) {
 		case ST_TRAVEL:
-			actor->UseExit(trap->GetGlobalID());
+			trap->AddTrigger(TriggerEntry(trigger_clicked, actor->GetGlobalID()));
+			// exit usage is handled by caller for now
+			// actor->UseExit(trap->GetGlobalID());
 			return false;
 		case ST_TRIGGER:
 			//the importer shouldn't load the script
@@ -1815,16 +1782,12 @@ bool GameControl::HandleActiveRegion(InfoPoint *trap, Actor * actor, Point &p)
 			//reset trap and deactivated flags
 			if (trap->Scripts[0]) {
 				if (!(trap->Flags&TRAP_DEACTIVATED) ) {
-					trap->LastTriggerObject = trap->LastTrigger = actor->GetGlobalID();
-					trap->ImmediateEvent();
+					trap->AddTrigger(TriggerEntry(trigger_clicked, actor->GetGlobalID()));
 					//directly feeding the event, even if there are actions in the queue
-					trap->Scripts[0]->Update();
-					trap->ProcessActions(true);
-					//if reset trap flag not set, deactivate it
-					//hmm, better not, info triggers don't deactivate themselves on click
-					//if (!(trap->Flags&TRAP_RESET)) {
-					//	trap->Flags|=TRAP_DEACTIVATED;
-					//}
+					//trap->Scripts[0]->Update();
+					// FIXME
+					trap->ExecuteScript(1);
+					trap->ProcessActions();
 				}
 			} else {
 				if (trap->overHeadText) {
@@ -1836,9 +1799,10 @@ bool GameControl::HandleActiveRegion(InfoPoint *trap, Actor * actor, Point &p)
 				}
 			}
 			if (trap->Flags&TRAP_USEPOINT) {
-				//overriding the target point
-				p = trap->UsePoint;
-				return false;
+				char Tmp[256];
+				sprintf(Tmp, "TriggerWalkTo(\"%s\")", trap->GetScriptName());
+				actor->AddAction(GenerateAction(Tmp));
+				return true;
 			}
 			return true;
 		default:;
@@ -1901,33 +1865,34 @@ void GameControl::OnMouseUp(unsigned short x, unsigned short y, unsigned short B
 	Map* area = game->GetCurrentArea( );
 	if (!area) return;
 
-#ifdef TOUCHSCREEN
-	touched=false;
-	if (scrolling) {
-		moveX = 0;
-		moveY = 0;
-		scrolling=false;
-		Video* video = core->GetVideoDriver();
-		video->SetDragCursor(NULL);
-		if (DrawSelectionRect) {
-			Actor** ab;
-			unsigned int count = area->GetActorInRect( ab, SelectionRect,true );
-			if (count != 0) {
-				for (i = 0; i < highlighted.size(); i++)
-					highlighted[i]->SetOver( false );
-				highlighted.clear();
-				game->SelectActor( NULL, false, SELECT_NORMAL );
-				for (i = 0; i < count; i++) {
-					// FIXME: should call handler only once
-					game->SelectActor( ab[i], true, SELECT_NORMAL );
+	if (touchScrollAreasEnabled) {
+		touched=false;
+		if (scrolling) {
+			moveX = 0;
+			moveY = 0;
+			scrolling=false;
+			Video* video = core->GetVideoDriver();
+			video->SetDragCursor(NULL);
+			if (DrawSelectionRect) {
+				Actor** ab;
+				unsigned int count = area->GetActorInRect( ab, SelectionRect,true );
+				if (count != 0) {
+					for (i = 0; i < highlighted.size(); i++)
+						highlighted[i]->SetOver( false );
+					highlighted.clear();
+					game->SelectActor( NULL, false, SELECT_NORMAL );
+					for (i = 0; i < count; i++) {
+						// FIXME: should call handler only once
+						game->SelectActor( ab[i], true, SELECT_NORMAL );
+					}
 				}
+				free( ab );
+				DrawSelectionRect = false;
 			}
-			free( ab );
-			DrawSelectionRect = false;
+			return;
 		}
-		return;
 	}
-#endif
+
 	if (DrawSelectionRect) {
 		Actor** ab;
 		unsigned int count = area->GetActorInRect( ab, SelectionRect,true );
@@ -1998,11 +1963,10 @@ void GameControl::OnMouseUp(unsigned short x, unsigned short y, unsigned short B
 				while(i--) {
 					game->selected[i]->UseExit(exitID);
 				}
-			} else {
-				if (HandleActiveRegion(overInfoPoint, pc, p)) {
-					core->SetEventFlag(EF_RESETTARGET);
-					return;
-				}
+			}
+			if (HandleActiveRegion(overInfoPoint, pc, p)) {
+				core->SetEventFlag(EF_RESETTARGET);
+				return;
 			}
 		}
 
@@ -2073,6 +2037,37 @@ void GameControl::OnMouseUp(unsigned short x, unsigned short y, unsigned short B
 	PerformActionOn(actor);
 }
 
+void GameControl::OnMouseWheelScroll(short x, short y)
+{
+	Region Viewport = core->GetVideoDriver()->GetViewport();
+	Game* game = core->GetGame();
+	Map *map = game->GetCurrentArea();
+	if (!map) return;
+	
+	Point mapsize = map->TMap->GetMapSize();
+	
+	if (Viewport.x + x <= 0) {
+		Viewport.x = 0;
+	}else if (Viewport.x + Viewport.w + x < mapsize.x) {
+		Viewport.x += x;
+	}
+	
+	if (Viewport.y + y <= 0) {
+		Viewport.y = 0;
+	}else if (Viewport.y + Viewport.h + y < mapsize.y) {
+		Viewport.y += y;
+	}
+	if (ScreenFlags & SF_LOCKSCROLL) {
+		moveX = 0;
+		moveY = 0;
+	} else {
+		// override any existing viewport moves which may be in progress
+		core->timer->SetMoveViewPort( Viewport.x, Viewport.y, 0, false );
+		// move it directly ourselves, since we might be paused
+		core->GetVideoDriver()->MoveViewportTo( Viewport.x, Viewport.y );
+	}	
+}
+
 void GameControl::PerformActionOn(Actor *actor)
 {
 	Game* game = core->GetGame();
@@ -2213,45 +2208,24 @@ void GameControl::OnSpecialKeyPress(unsigned char Key)
 		}
 		return; //don't accept keys in dialog
 	}
-	Region Viewport = core->GetVideoDriver()->GetViewport();
 	Game *game = core->GetGame();
 	if (!game) return;
-	Map *map = game->GetCurrentArea();
-	if (!map) return;
-
-	Point mapsize = map->TMap->GetMapSize();
 	int partysize = game->GetPartySize(false);
+	
 	int pm;
 	char tmpstr[10];
-
 	switch (Key) {
 		case GEM_LEFT:
-			if (Viewport.x > 63)
-				Viewport.x -= 64;
-			else
-				Viewport.x = 0;
+			OnMouseWheelScroll(-64, 0);
 			break;
 		case GEM_UP:
-			if (Viewport.y > 63)
-				Viewport.y -= 64;
-			else
-				Viewport.y = 0;
+			OnMouseWheelScroll(0, -64);
 			break;
 		case GEM_DOWN:
-			if (Viewport.y + Viewport.h + 64 < mapsize.y)
-				Viewport.y += 64;
-			else {
-				Viewport.y = mapsize.y - Viewport.h;
-				if (Viewport.y<0) Viewport.y=0;
-			}
+			OnMouseWheelScroll(0, 64);
 			break;
 		case GEM_RIGHT:
-			if (Viewport.x + Viewport.w + 64 < mapsize.x)
-				Viewport.x += 64;
-			else {
-				Viewport.x = mapsize.x - Viewport.w;
-				if (Viewport.x<0) Viewport.x=0;
-			}
+			OnMouseWheelScroll(64, 0);
 			break;
 		case GEM_ALT:
 			DebugFlags |= DEBUG_SHOW_CONTAINERS;
@@ -2286,15 +2260,6 @@ void GameControl::OnSpecialKeyPress(unsigned char Key)
 		default:
 			return;
 	}
-	if (ScreenFlags & SF_LOCKSCROLL) {
-		moveX = 0;
-		moveY = 0;
-	} else {
-		// override any existing viewport moves which may be in progress
-		core->timer->SetMoveViewPort( Viewport.x, Viewport.y, 0, false );
-		// move it directly ourselves, since we might be paused
-		core->GetVideoDriver()->MoveViewportTo( Viewport.x, Viewport.y );
-	}
 }
 
 void GameControl::CalculateSelection(const Point &p)
@@ -2386,8 +2351,8 @@ void GameControl::HandleWindowHide(const char *WindowName, const char *WindowPos
 				}
 				return;
 			}
-			printMessage("GameControl", "Invalid Window Index: ", LIGHT_RED);
-			printf("%s:%u\n",WindowName, index);
+			printMessage("GameControl", "Invalid Window Index: %s:%u\n", LIGHT_RED,
+				WindowName, index);
 		}
 	}
 }
@@ -2439,8 +2404,8 @@ void GameControl::HandleWindowReveal(const char *WindowName, const char *WindowP
 				}
 				return;
 			}
-			printMessage("GameControl", "Invalid Window Index ", LIGHT_RED);
-			printf("%s:%u\n",WindowName, index);
+			printMessage("GameControl", "Invalid Window Index %s:%u\n", LIGHT_RED,
+				WindowName, index);
 		}
 	}
 }
@@ -2694,7 +2659,7 @@ Sprite2D* GameControl::GetScreenshot(bool show_gui)
 //copies a downscaled screenshot into a sprite for save game preview
 Sprite2D* GameControl::GetPreview()
 {
-	// We get preview by first taking a screenshot of size 640x405,
+	// We get preview by first taking a screenshot of quintuple size of the preview control size (a few pixels bigger only in pst),
 	// centered in the display. This is to get a decent picture for
 	// higher screen resolutions.
 	// FIXME: how do orig games solve that?
@@ -2707,27 +2672,20 @@ Sprite2D* GameControl::GetPreview()
 	if (x < 0) {
 		x = 0;
 	} else {
-		w = 640;
+		w = 515;
 	}
 
 	if (y < 0) {
 		y = 0;
 	} else {
-		h = 405;
+		h = 385;
 	}
 
 	if (!x)
 		y = 0;
 
-	int hf = HideGUI ();
-	signed char v = Owner->Visible;
-	Owner->Visible = WINDOW_VISIBLE;
 	Draw (0, 0);
-	Owner->Visible = v;
 	Sprite2D *screenshot = video->GetScreenshot( Region(x, y, w, h) );
-	if (hf) {
-		UnhideGUI ();
-	}
 	core->DrawWindows();
 
 	Sprite2D* preview = video->SpriteScaleDown ( screenshot, 5 );
