@@ -32,6 +32,8 @@
 #include "Palette.h"
 #include "Variables.h"
 #include "Video.h"
+#include "GUI/EventMgr.h"
+#include "GUI/Window.h"
 #include "Scriptable/Actor.h"
 
 #include <cstdio>
@@ -126,7 +128,7 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 	{
 		unsigned long thisTime;
 
-		GetTime( thisTime);
+		thisTime = GetTickCount();
 		if (thisTime>starttime) {
 			starttime = thisTime+ticks;
 			smooth--;
@@ -177,9 +179,8 @@ void TextArea::Draw(unsigned short x, unsigned short y)
 		for (size_t i = 0; i < linesize; i++) {
 			if (strnicmp( "[s=", lines[i], 3 ) == 0) {
 				int tlen;
-				unsigned long idx, acolor, bcolor;
-				char* rest;
-				idx = strtoul( lines[i] + 3, &rest, 0 );
+				unsigned long acolor, bcolor;
+				char* rest = strchr( lines[i] + 3, ',' );
 				if (*rest != ',')
 					goto notmatched;
 				acolor = strtoul( rest + 1, &rest, 16 );
@@ -297,36 +298,26 @@ int TextArea::SetScrollBar(Control* ptr)
 }
 
 /** Sets the Actual Text */
-int TextArea::SetText(const char* text, int pos)
+void TextArea::SetText(const char* text)
 {
-	if (pos==0) {
-		if (!text[0]) {
-			lines.clear();
-			lrows.clear();
-		}
+	if (!text[0]) {
+		Clear();
+	}
 
-		if (lines.size() == 0) {
-			pos = -1;
-		}
-	}
-	if (pos >= ( int ) lines.size()) {
-		return -1;
-	}
 	int newlen = ( int ) strlen( text );
 
-	if (pos == -1) {
+	if (lines.size() == 0) {
 		char* str = (char *) malloc( newlen + 1 );
 		memcpy( str, text, newlen + 1 );
 		lines.push_back( str );
 		lrows.push_back( 0 );
 	} else {
-		lines[pos] = (char *) realloc( lines[pos], newlen + 1 );
-		memcpy( lines[pos], text, newlen + 1 );
+		lines[0] = (char *) realloc( lines[0], newlen + 1 );
+		memcpy( lines[0], text, newlen + 1 );
 	}
 	CurPos = newlen;
 	CurLine = lines.size()-1;
 	UpdateControls();
-	return 0;
 }
 
 void TextArea::SetMinRow(bool enable)
@@ -479,7 +470,7 @@ void TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 			Owner->Invalidate();
 			Changed = true;
 			int len = GetRowLength(CurLine);
-			//printf("len: %d Before: %s\n",len, lines[CurLine]);
+			//print("len: %d Before: %s\n",len, lines[CurLine]);
 			lines[CurLine] = (char *) realloc( lines[CurLine], len + 2 );
 			for (int i = len; i > CurPos; i--) {
 				lines[CurLine][i] = lines[CurLine][i - 1];
@@ -487,7 +478,7 @@ void TextArea::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 			lines[CurLine][CurPos] = Key;
 			lines[CurLine][len + 1] = 0;
 			CurPos++;
-			//printf("pos: %d After: %s\n",CurPos, lines[CurLine]);
+			//print("pos: %d After: %s\n",CurPos, lines[CurLine]);
 			CalcRowCount();
 			RunEventHandler( TextAreaOnChange );
 		}
@@ -577,7 +568,7 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 			break;
 		case GEM_DELETE:
 			len = GetRowLength(CurLine);
-			//printf("len: %d Before: %s\n",len, lines[CurLine]);
+			//print("len: %d Before: %s\n",len, lines[CurLine]);
 			if (CurPos>=len) {
 				//TODO: merge next line
 				break;
@@ -586,12 +577,12 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 			for (i = CurPos; i < len; i++) {
 				lines[CurLine][i] = lines[CurLine][i + 1];
 			}
-			//printf("pos: %d After: %s\n",CurPos, lines[CurLine]);
+			//print("pos: %d After: %s\n",CurPos, lines[CurLine]);
 			break;
 		case GEM_BACKSP:
 			len = GetRowLength(CurLine);
 			if (CurPos != 0) {
-				//printf("len: %d Before: %s\n",len, lines[CurLine]);
+				//print("len: %d Before: %s\n",len, lines[CurLine]);
 				if (len<1) {
 					break;
 				}
@@ -601,15 +592,15 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 				}
 				lines[CurLine][len - 1] = 0;
 				CurPos--;
-				//printf("pos: %d After: %s\n",CurPos, lines[CurLine]);
+				//print("pos: %d After: %s\n",CurPos, lines[CurLine]);
 			} else {
 				if (CurLine) {
 					//TODO: merge lines
 					int oldline = CurLine;
 					CurLine--;
 					int old = GetRowLength(CurLine);
-					//printf("len: %d Before: %s\n",old, lines[CurLine]);
-					//printf("len: %d Before: %s\n",len, lines[oldline]);
+					//print("len: %d Before: %s\n",old, lines[CurLine]);
+					//print("len: %d Before: %s\n",len, lines[oldline]);
 					lines[CurLine] = (char *) realloc (lines[CurLine], len+old);
 					memcpy(lines[CurLine]+old, lines[oldline],len);
 					free(lines[oldline]);
@@ -617,13 +608,13 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 					lines.erase(lines.begin()+oldline);
 					lrows.erase(lrows.begin()+oldline);
 					CurPos = old;
-					//printf("pos: %d len: %d After: %s\n",CurPos, GetRowLength(CurLine), lines[CurLine]);
+					//print("pos: %d len: %d After: %s\n",CurPos, GetRowLength(CurLine), lines[CurLine]);
 				}
 			}
 			break;
 		 case GEM_RETURN:
 			//add an empty line after CurLine
-			//printf("pos: %d Before: %s\n",CurPos, lines[CurLine]);
+			//print("pos: %d Before: %s\n",CurPos, lines[CurLine]);
 			lrows.insert(lrows.begin()+CurLine, 0);
 			len = GetRowLength(CurLine);
 			//copy the text after the cursor into the new line
@@ -637,8 +628,8 @@ void TextArea::OnSpecialKeyPress(unsigned char Key)
 			//move cursor to next line beginning
 			CurLine++;
 			CurPos=0;
-			//printf("len: %d After: %s\n",GetRowLength(CurLine-1), lines[CurLine-1]);
-			//printf("len: %d After: %s\n",GetRowLength(CurLine), lines[CurLine]);
+			//print("len: %d After: %s\n",GetRowLength(CurLine-1), lines[CurLine-1]);
+			//print("len: %d After: %s\n",GetRowLength(CurLine), lines[CurLine]);
 			break;
 	}
 	CalcRowCount();
@@ -780,7 +771,7 @@ void TextArea::OnMouseOver(unsigned short /*x*/, unsigned short y)
 			if (seltext != (int) i)
 				core->RedrawAll();
 			seltext = ( int ) i;
-			//printf("CtrlId = 0x%08lx, seltext = %d, rows = %d, row = %d, r = %d\n", ControlID, i, rows, row, r);
+			//print("CtrlId = 0x%08lx, seltext = %d, rows = %d, row = %d, r = %d\n", ControlID, i, rows, row, r);
 			return;
 		}
 	}
@@ -788,7 +779,7 @@ void TextArea::OnMouseOver(unsigned short /*x*/, unsigned short y)
 		core->RedrawAll();
 	}
 	seltext = -1;
-	//printf("CtrlId = 0x%08lx, seltext = %d, rows %d, row %d, r = %d\n", ControlID, seltext, rows, row, r);
+	//print("CtrlId = 0x%08lx, seltext = %d, rows %d, row %d, r = %d\n", ControlID, seltext, rows, row, r);
 }
 
 /** Mouse Button Up */
@@ -822,13 +813,25 @@ void TextArea::OnMouseUp(unsigned short x, unsigned short y, unsigned short /*Bu
 	RunEventHandler( TextAreaOnChange );
 }
 
-/** Copies the current TextArea content to another TextArea control */
-void TextArea::CopyTo(TextArea* ta)
+void TextArea::SetText(const std::vector<char*>& text)
 {
-	ta->Clear();
-	for (size_t i = 0; i < lines.size(); i++) {
-		ta->SetText( lines[i], -1 );
+	Clear();
+	for (size_t i = 0; i < text.size(); i++) {
+		int newlen = strlen(text[i]);
+		char* str = (char *) malloc(newlen + 1);
+		memcpy(str, text[i], newlen + 1);
+		lines.push_back(str);
+		lrows.push_back(0);
+		CurPos = newlen;
 	}
+	CurLine = lines.size() - 1;
+	UpdateControls();
+}
+
+/** Copies the current TextArea content to another TextArea control */
+void TextArea::CopyTo(TextArea *ta)
+{
+	ta->SetText(lines);
 }
 
 void TextArea::RedrawTextArea(const char* VariableName, unsigned int Sum)
@@ -939,7 +942,7 @@ void TextArea::SetupScroll(unsigned long tck)
 	}
 	i = (unsigned int) lines.size();
 	Flags |= IE_GUI_TEXTAREA_SMOOTHSCROLL;
-	GetTime( starttime );
+	starttime = GetTickCount();
 	if (RunEventHandler( TextAreaOutOfText )) {
 		//event handler destructed this object?
 		return;
@@ -977,3 +980,11 @@ void TextArea::OnMouseDown(unsigned short /*x*/, unsigned short /*y*/, unsigned
 		}
 	}
 }
+
+void TextArea::SetFocus(bool focus)
+{
+	Control::SetFocus(focus);
+	if (hasFocus && Flags & IE_GUI_TEXTAREA_EDITABLE) {
+		core->GetVideoDriver()->ShowSoftKeyboard();
+	}
+}
