@@ -71,7 +71,9 @@ public:
 	/** Set the TextArea value to the line number containing the string parameter */
 	void SelectText(const char *select);
 	/** Sets the Actual Text */
-	int SetText(const char* text, int pos = 0);
+	void SetText(const char* text);
+	/** Sets text */
+	void SetText(const std::vector<char*>& text);
 	/** Clears the textarea */
 	void Clear();
 	/** Discards scrolled out lines from the textarea */
@@ -168,6 +170,7 @@ public: //Events
 		unsigned short Mod);
 	/** Set handler for specified event */
 	bool SetEvent(int eventType, EventHandler handler);
+	void SetFocus(bool focus);
 	/** OnChange Scripted Event Function Name */
 	EventHandler TextAreaOnChange;
 	/** OutOfText Scripted Event Function Name */
