@@ -29,10 +29,13 @@
 #include "DisplayMessage.h"
 #include "GameData.h"
 #include "Interface.h"
+#include "IniSpawn.h"
 #include "MapMgr.h"
 #include "MusicMgr.h"
 #include "Particles.h"
+#include "PluginMgr.h"
 #include "ScriptEngine.h"
+#include "TableMgr.h"
 #include "GameScript/GameScript.h"
 #include "GUI/GameControl.h"
 #include "System/DataStream.h"
@@ -149,7 +152,7 @@ Game::~Game(void)
 	}
 }
 
-bool IsAlive(Actor *pc)
+static bool IsAlive(Actor *pc)
 {
 	if (pc->GetStat(IE_STATE_ID)&STATE_DEAD) {
 		return false;
@@ -312,6 +315,8 @@ void Game::ConsolidateParty()
 	}
 	for ( m = PCs.begin(); m != PCs.end(); ++m) {
 		(*m)->RefreshEffects(NULL);
+		//TODO: how to set up bardsongs
+		(*m)->SetModalSpell((*m)->ModalState, 0);
 	}
 }
 
@@ -369,8 +374,7 @@ void Game::InitActorPos(Actor *actor)
 	AutoTable strta("startpos");
 
 	if (!start || !strta) {
-		printMessage("Game","Game is missing character start data.\n",RED);
-		abort();
+		error("Game", "Game is missing character start data.\n");
 	}
 	// 0 - single player, 1 - tutorial, 2 - expansion
 	ieDword playmode = 0;
@@ -475,7 +479,7 @@ void Game::SetHotKey(unsigned long Key)
 		Actor *actor = *m;
 
 		if (actor->IsSelected()) {
-			actor->HotKey = (ieDword) Key;
+			actor->AddTrigger(TriggerEntry(trigger_hotkey, (ieDword) Key));
 		}
 	}
 }
@@ -769,7 +773,7 @@ int Game::LoadMap(const char* ResRef, bool loadscreen)
 	if (!ds) {
 		goto failedload;
 	}
-	if(!mM->Open( ds, true )) {
+	if(!mM->Open(ds)) {
 		goto failedload;
 	}
 	newMap = mM->GetMap(ResRef, IsDay());
@@ -879,7 +883,11 @@ bool Game::AddJournalEntry(ieStrRef strref, int Section, int Group)
 			je->Group = (ieByte) Group;
 			ieDword chapter = 0;
 			locals->Lookup("CHAPTER", chapter);
-			je->Chapter = (ieByte) chapter;
+			if (core->HasFeature(GF_NO_NEW_VARIABLES)) {
+				je->Chapter = 0;
+			} else {
+				je->Chapter = (ieByte) chapter;
+			}
 			je->GameTime = GameTime;
 			return true;
 		}
@@ -888,7 +896,11 @@ bool Game::AddJournalEntry(ieStrRef strref, int Section, int Group)
 	je->GameTime = GameTime;
 	ieDword chapter = 0;
 	locals->Lookup("CHAPTER", chapter);
-	je->Chapter = (ieByte) chapter;
+	if (core->HasFeature(GF_NO_NEW_VARIABLES)) {
+		je->Chapter = 0;
+	} else {
+		je->Chapter = (ieByte) chapter;
+	}
 	je->unknown09 = 0;
 	je->Section = (ieByte) Section;
 	je->Group = (ieByte) Group;
@@ -1021,7 +1033,7 @@ int Game::GetXPFromCR(int cr)
 		if (cr>=MAX_CRLEVEL) {
 			cr=MAX_CRLEVEL-1;
 		}
-		printf("Challenge Rating: %d, party level: %d ", cr, level);
+		print("Challenge Rating: %d, party level: %d ", cr, level);
 		return crtable[level][cr];
 	}
 	printMessage("Game","Cannot find moncrate.2da!\n", LIGHT_RED);
@@ -1097,7 +1109,7 @@ bool Game::EveryoneNearPoint(Map *area, const Point &p, int flags) const
 			return false;
 		}
 		if (Distance(p,PCs[i])>MAX_TRAVELING_DISTANCE) {
-printf("Actor %s is not near!\n", PCs[i]->LongName);
+			print("Actor %s is not near!\n", PCs[i]->LongName);
 			return false;
 		}
 	}
@@ -1124,17 +1136,6 @@ void Game::PartyMemberDied(Actor *actor)
 	}
 }
 
-//reports if someone died
-int Game::PartyMemberDied() const
-{
-	for (unsigned int i=0; i<PCs.size(); i++) {
-		if (PCs[i]->GetInternalFlag()&IF_JUSTDIED) {
-			return i;
-		}
-	}
-	return -1;
-}
-
 void Game::IncrementChapter()
 {
 	//chapter first set to 0 (prologue)
@@ -1240,33 +1241,13 @@ bool Game::PartyOverflow() const
 	return (PCs.size()>partysize);
 }
 
-bool Game::PCInCombat(Actor* actor) const
-{
-	if (!CombatCounter) {
-		return false;
-	}
-
-	if (actor->LastTarget) {
-		return true;
-	}
-	if (AttackersOf(actor->GetGlobalID(), actor->GetCurrentArea())) {
-		return true;
-	}
-	return false;
-}
-
 bool Game::AnyPCInCombat() const
 {
 	if (!CombatCounter) {
 		return false;
 	}
 
-	for (unsigned int i=0; i<PCs.size(); i++) {
-		if (PCInCombat (PCs[i])) {
-			return true;
-		}
-	}
-	return false;
+	return true;
 }
 
 //returns true if the protagonist (or the whole party died)
@@ -1305,34 +1286,13 @@ bool Game::EveryoneDead() const
 
 void Game::UpdateScripts()
 {
-	ExecuteScript( 1 );
-	ProcessActions(false);
+	Update();
 	size_t idx;
 
-	bool PartyAttack = false;
+	PartyAttack = false;
 
 	for (idx=0;idx<Maps.size();idx++) {
 		Maps[idx]->UpdateScripts();
-		size_t acnt=Attackers.size();
-		while(acnt--) {
-			Actor *actor = Maps[idx]->GetActorByGlobalID(Attackers[acnt]);
-			if (actor) {
-				if ( !Maps[idx]->GetActorByGlobalID(actor->LastTarget) ) {
-					//Actor's target left area
-					OutAttack(Attackers[acnt]);
-					continue;
-				} else {
-					//each attacker handles their own round initiation
-					actor->InitRound(GameTime);
-					if (actor->InParty) {
-						PartyAttack = true;
-					}
-				}
-			} else {
-				//Attacker is gone from area
-				OutAttack(Attackers[acnt]);
-			}
-		}
 	}
 
 	if (PartyAttack) {
@@ -1564,7 +1524,19 @@ void Game::RestParty(int checks, int dream, int hp)
 void Game::TimeStop(Actor* owner, ieDword end)
 {
 	timestop_owner=owner;
-	timestop_end=GameTime+end;
+	timestop_end=end;
+}
+
+// check if the passed actor is a victim of timestop
+bool Game::TimeStoppedFor(const Actor* target)
+{
+	if (!timestop_owner) {
+		return false;
+	}
+	if (target == timestop_owner || target->GetStat(IE_DISABLETIMESTOP)) {
+		return false;
+	}
+	return true;
 }
 
 //recalculate the party's infravision state
@@ -1589,7 +1561,6 @@ void Game::Infravision()
 //returns the colour which should be applied onto the whole game area viewport
 //this is based on timestop, dream area, weather, daytime
 
-static const Color TimeStopTint={0xe0,0xe0,0xe0,0x20}; //greyscale
 static const Color DreamTint={0xf0,0xe0,0xd0,0x10};    //light brown scale
 static const Color NightTint={0x80,0x80,0xe0,0x40};    //dark, bluish
 static const Color DuskTint={0xe0,0x80,0x80,0x40};     //dark, reddish
@@ -1598,9 +1569,6 @@ static const Color DarkTint={0x80,0x80,0xe0,0x10};     //slightly dark bluish
 
 const Color *Game::GetGlobalTint() const
 {
-	if (timestop_end>GameTime) {
-		return &TimeStopTint;
-	}
 	Map *map = GetCurrentArea();
 	if (!map) return NULL;
 	if (map->AreaFlags&AF_DREAM) {
@@ -1638,28 +1606,6 @@ bool Game::IsDay()
 	return true;
 }
 
-void Game::InAttack(ieDword globalID)
-{
-	std::vector< ieDword>::const_iterator idx;
-
-	for(idx=Attackers.begin(); idx!=Attackers.end();idx++) {
-		if (*idx==globalID) return;
-	}
-	Attackers.push_back(globalID);
-}
-
-void Game::OutAttack(ieDword globalID)
-{
-	std::vector< ieDword>::iterator idx;
-
-	for(idx=Attackers.begin(); idx!=Attackers.end();idx++) {
-		if (*idx==globalID) {
-			Attackers.erase(idx);
-			break;
-		}
-	}
-}
-
 void Game::ChangeSong(bool always, bool force)
 {
 	int Song;
@@ -1678,27 +1624,8 @@ void Game::ChangeSong(bool always, bool force)
 	area->PlayAreaSong( Song, always, force );
 }
 
-int Game::AttackersOf(ieDword globalID, Map *area) const
-{
-	if (!area) {
-		return 0;
-	}
-	std::vector< ieDword>::const_iterator idx;
-
-	int cnt = 0;
-	for(idx=Attackers.begin(); idx!=Attackers.end();idx++) {
-		Actor * actor = area->GetActorByGlobalID(*idx);
-		if (actor) {
-			if (actor->LastTarget==globalID) {
-				cnt++;
-			}
-		}
-	}
-	return cnt;
-}
-
 /* this method redraws weather. If update is false,
-then the weather particles won't change (game paused)
+   then the weather particles won't change (game paused)
 */
 void Game::DrawWeather(const Region &screen, bool update)
 {
@@ -1798,31 +1725,21 @@ void Game::DebugDump()
 {
 	size_t idx;
 
-	printf("Currently loaded areas:\n");
+	print("Currently loaded areas:\n");
 	for(idx=0;idx<Maps.size();idx++) {
 		Map *map = Maps[idx];
 
-		printf("%s\n",map->GetScriptName());
-	}
-	printf("Current area: %s   Previous area: %s\n", CurrentArea, PreviousArea);
-	printf("Global script: %s\n", Scripts[0]->GetName());
-	printf("CombatCounter: %d\n", (int) CombatCounter);
-	printf("Attackers count: %d\n", (int) Attackers.size());
-	for(idx=0;idx<Attackers.size(); idx++) {
-		Actor *actor = GetActorByGlobalID(Attackers[idx]);
-		if (!actor) {
-			printf("Name: ???\n");
-			continue;
-		}
-		Actor *whom = GetActorByGlobalID(actor->LastTarget);
-		printf("Name: %s Attacking : %s\n", actor->ShortName, whom?whom->ShortName:"???");
+		print("%s\n",map->GetScriptName());
 	}
+	print("Current area: %s   Previous area: %s\n", CurrentArea, PreviousArea);
+	print("Global script: %s\n", Scripts[0]->GetName());
+	print("CombatCounter: %d\n", (int) CombatCounter);
 
-	printf("Party size: %d\n", (int) PCs.size());
+	print("Party size: %d\n", (int) PCs.size());
 	for(idx=0;idx<PCs.size();idx++) {
 		Actor *actor = PCs[idx];
 
-		printf("Name: %s Order %d %s\n",actor->ShortName, actor->InParty, actor->Selected?"x":"-");
+		print("Name: %s Order %d %s\n",actor->ShortName, actor->InParty, actor->Selected?"x":"-");
 	}
 }
 
@@ -1846,3 +1763,7 @@ ieByte *Game::AllocateMazeData()
 	return mazedata;
 }
 
+bool Game::IsTimestopActive()
+{
+	return timestop_end > GameTime;
+}
