@@ -34,13 +34,16 @@ class Game;
 #include "ie_types.h"
 
 #include "Callback.h"
-#include "Map.h"
+#include "Scriptable/Scriptable.h"
+#include "Scriptable/PCStatStruct.h"
 #include "Variables.h"
-#include "Scriptable/Actor.h"
 
 #include <vector>
 
+class Actor;
+class Map;
 class Particles;
+class TableMgr;
 
 //the size of the bestiary register
 #define BESTIARY_SIZE 260
@@ -236,7 +239,6 @@ private:
 	std::vector< GAMLocationEntry*> savedpositions;
 	std::vector< GAMLocationEntry*> planepositions;
 	std::vector< char*> mastarea;
-	std::vector< ieDword> Attackers;
 	CRRow *crtable;
 	ieResRef restmovies[8];
 	ieResRef daymovies[8];
@@ -286,6 +288,7 @@ public:
 	EventHandler event_handler; //like in Control
 	bool hasInfra;
 	bool familiarBlock;
+	bool PartyAttack;
 private:
 	/** reads the challenge rating table */
 	void LoadCRTable();
@@ -354,9 +357,6 @@ public:
 	Actor* GetNPC(unsigned int Index);
 	void SwapPCs(unsigned int Index1, unsigned int Index2);
 	bool IsDay();
-	void InAttack(ieDword globalID);
-	void OutAttack(ieDword globalID);
-	int AttackersOf(ieDword globalID, Map *area) const;
 
 	//journal entries
 	/** Deletes one or all journal entries if strref is -1 */
@@ -409,9 +409,6 @@ public:
 		}
 		return Formations[WhichFormation];
 	}
-	size_t GetAttackerCount() const {
-		return Attackers.size();
-	}
 
 	/** converts challenge rating to xp */
 	int GetXPFromCR(int cr);
@@ -419,8 +416,6 @@ public:
 	void ShareXP(int XP, int flags);
 	/** returns true if we should start the party overflow window */
 	bool PartyOverflow() const;
-	/** returns true if actor is an attacker or being attacked */
-	bool PCInCombat(Actor *actor) const;
 	/** returns true if any pc is attacker or being attacked */
 	bool AnyPCInCombat() const;
 	/** returns true if the party death condition is true */
@@ -428,8 +423,6 @@ public:
 	/** returns true if no one moves */
 	bool EveryoneStopped() const;
 	bool EveryoneNearPoint(Map *map, const Point &p, int flags) const;
-	/** returns true if a PC just died */
-	int PartyMemberDied() const;
 	/** a party member just died now */
 	void PartyMemberDied(Actor *);
 	/** Increments chapter variable and refreshes kill stats */
@@ -457,6 +450,8 @@ public:
 	void RestParty(int checks, int dream, int hp);
 	/** timestop effect initiated by actor */
 	void TimeStop(Actor *actor, ieDword end);
+	/** check if the passed actor is a victim of timestop */
+	bool TimeStoppedFor(const Actor* target=NULL);
 	/** updates the infravision info */
 	void Infravision();
 	/** gets the colour which should be applied over the game area,
@@ -476,6 +471,8 @@ public:
 	Actor *GetActorByGlobalID(ieDword objectID);
 	/** Allocates maze data */
 	ieByte *AllocateMazeData();
+	/** Checks if any timestop effects are active */
+	bool IsTimestopActive();
 private:
 	bool DetermineStartPosType(const TableMgr *strta);
 	ieResRef *GetDream(Map *area);
