@@ -24,11 +24,19 @@
 #include "SClassID.h"
 #include "exports.h"
 #include "ie_types.h"
+#include "iless.h"
 
 #include "Cache.h"
 #include "Holder.h"
 #include "ResourceManager.h"
 
+#include <map>
+#include <vector>
+
+#ifdef _MSC_VER // No SFINAE
+#include "TableMgr.h"
+#endif
+
 class Actor;
 struct Effect;
 class Factory;
@@ -38,6 +46,7 @@ class ScriptedAnimation;
 class Spell;
 class Sprite2D;
 class TableMgr;
+class Store;
 
 struct Table {
 	Holder<TableMgr> tm;
@@ -77,7 +86,6 @@ public:
 	Item* GetItem(const ieResRef resname);
 	void FreeItem(Item const *itm, const ieResRef name, bool free=false);
 	Spell* GetSpell(const ieResRef resname, bool silent=false);
-	Spell* GetUncachedSpell(const ieResRef resname, bool silent=false);
 	void FreeSpell(Spell *spl, const ieResRef name, bool free=false);
 	Effect* GetEffect(const ieResRef resname);
 	void FreeEffect(Effect *eff, const ieResRef name, bool free=false);
@@ -91,6 +99,12 @@ public:
 	/** returns factory resource, currently works only with animations */
 	void* GetFactoryResource(const char* resname, SClass_ID type,
 		unsigned char mode = IE_NORMAL, bool silent=false);
+
+	Store* GetStore(const ieResRef ResRef);
+	/// Saves a store to the cache and frees it.
+	void SaveStore(Store* store);
+	/// Saves all stores in the cache
+	void SaveAllStores();
 private:
 	Cache ItemCache;
 	Cache SpellCache;
@@ -98,6 +112,8 @@ private:
 	Cache PaletteCache;
 	Factory* factory;
 	std::vector<Table> tables;
+	typedef std::map<const char*, Store*, iless> StoreMap;
+	StoreMap stores;
 };
 
 extern GEM_EXPORT GameData * gamedata;
