@@ -48,7 +48,7 @@ class GameScript;
 #define DS_SPEECH  64
 #define DS_AREA    128
 
-//verbal constant (bg2), we need a lookup table for other games
+//verbal constant (bg2), we have a lookup table (vcremap) for other games
 #define VB_PANIC     1
 #define VB_HAPPY     2
 #define VB_UNHAPPY   3
@@ -59,6 +59,7 @@ class GameScript;
 #define VB_DAMAGE    18
 #define VB_DIE       19
 #define VB_SELECT    26
+#define VB_COMMAND   32
 #define VB_INSULT    44
 #define VB_COMPLIMENT 47
 #define VB_SPECIAL   50
@@ -66,8 +67,9 @@ class GameScript;
 #define VB_REACT_S   54
 #define VB_RESP_COMP 55
 #define VB_RESP_INS  58
-#define VB_HOSTILE   59
-#define VB_DIALOG    60
+#define VB_HOSTILE   61
+#define VB_DIALOG    62
+#define VB_SELECT_RARE 63
 #define VB_CRITHIT   65
 #define VB_CRITMISS  66
 #define VB_TIMMUNE   67
@@ -106,7 +108,7 @@ class GameScript;
 #define GSASSERT(f,c) \
 	if(!(f))  \
 	{  \
-		printf("Assertion failed: %s [0x%08lX] Line %d",#f, c, __LINE__); \
+		print("Assertion failed: %s [0x%08lX] Line %d",#f, c, __LINE__); \
 		abort(); \
 	}
 
@@ -146,7 +148,7 @@ class GEM_EXPORT Object {
 public:
 	Object()
 	{
-		objectName[0] = 0;
+		memset( objectName, 0, 65 );
 
 		memset( objectFields, 0, MAX_OBJECT_FIELDS * sizeof( int ) );
 		memset( objectFilters, 0, MAX_NESTING * sizeof( int ) );
@@ -171,19 +173,19 @@ public:
 
 		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
 		if(objectName[0]) {
-			printf("Object: %s\n",objectName);
+			print("Object: %s\n",objectName);
 			return;
 		}
-		printf("IDS Targeting: ");
+		print("IDS Targeting: ");
 		for(i=0;i<MAX_OBJECT_FIELDS;i++) {
-			printf("%d ",objectFields[i]);
+			print("%d ",objectFields[i]);
 		}
-		printf("\n");
-		printf("Filters: ");
+		print("\n");
+		print("Filters: ");
 		for(i=0;i<MAX_NESTING;i++) {
-			printf("%d ",objectFilters[i]);
+			print("%d ",objectFilters[i]);
 		}
-		printf("\n");
+		print("\n");
 	}
 
 	void Release()
@@ -192,19 +194,21 @@ public:
 		canary = 0xdddddddd;
 		delete this;
 	}
-	bool ReadyToDie();
+	bool isNull();
 };
 
 class GEM_EXPORT Trigger {
 public:
 	Trigger()
 	{
+		triggerID = 0;
 		flags = 0;
 		objectParameter = NULL;
-		string0Parameter[0] = 0;
-		string1Parameter[0] = 0;
+		memset(string0Parameter, 0, 65);
+		memset(string1Parameter, 0, 65);
 		int0Parameter = 0;
 		int1Parameter = 0;
+		int2Parameter = 0;
 		pointParameter.null();
 		canary = (unsigned long) 0xdeadbeef;
 	}
@@ -232,17 +236,17 @@ public:
 	void Dump()
 	{
 		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
-		printf ("Trigger: %d\n", triggerID);
-		printf ("Int parameters: %d %d %d\n", int0Parameter, int1Parameter, int2Parameter);
-		printf ("Point: [%d.%d]\n", pointParameter.x, pointParameter.y);
-		printf ("String0: %s\n", string0Parameter);
-		printf ("String1: %s\n", string1Parameter);
+		print ("Trigger: %d\n", triggerID);
+		print ("Int parameters: %d %d %d\n", int0Parameter, int1Parameter, int2Parameter);
+		print ("Point: [%d.%d]\n", pointParameter.x, pointParameter.y);
+		print ("String0: %s\n", string0Parameter);
+		print ("String1: %s\n", string1Parameter);
 		if (objectParameter) {
 			objectParameter->Dump();
 		} else {
-			printf("No object\n");
+			print("No object\n");
 		}
-		printf("\n");
+		print("\n");
 	}
 
 	void Release()
@@ -290,8 +294,8 @@ public:
 		objects[0] = NULL;
 		objects[1] = NULL;
 		objects[2] = NULL;
-		string0Parameter[0] = 0;
-		string1Parameter[0] = 0;
+		memset(string0Parameter, 0, 65);
+		memset(string1Parameter, 0, 65);
 		int0Parameter = 0;
 		pointParameter.null();
 		int1Parameter = 0;
@@ -334,25 +338,25 @@ public:
 		int i;
 
 		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
-		printf("Int0: %d, Int1: %d, Int2: %d\n",int0Parameter, int1Parameter, int2Parameter);
-		printf("String0: %s, String1: %s\n", string0Parameter?string0Parameter:"<NULL>", string1Parameter?string1Parameter:"<NULL>");
+		print("Int0: %d, Int1: %d, Int2: %d\n",int0Parameter, int1Parameter, int2Parameter);
+		print("String0: %s, String1: %s\n", string0Parameter?string0Parameter:"<NULL>", string1Parameter?string1Parameter:"<NULL>");
 		for (i=0;i<3;i++) {
 			if (objects[i]) {
-				printf( "%d. ",i+1);
+				print( "%d. ",i+1);
 				objects[i]->Dump();
 			} else {
-				printf( "%d. Object - NULL\n",i+1);
+				print( "%d. Object - NULL\n",i+1);
 			}
 		}
 
-		printf("RefCount: %d\n", RefCount);
+		print("RefCount: %d\n", RefCount);
 	}
 
 	void Release()
 	{
 		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
 		if (!RefCount) {
-			printf( "WARNING!!! Double Freeing in %s: Line %d\n", __FILE__,
+			print( "WARNING!!! Double Freeing in %s: Line %d\n", __FILE__,
 				__LINE__ );
 			abort();
 		}
@@ -367,7 +371,7 @@ public:
 		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
 		RefCount++;
 		if (RefCount >= 65536) {
-			printf( "Refcount increased to: %d in action %d\n", RefCount,
+			print( "Refcount increased to: %d in action %d\n", RefCount,
 				actionID );
 			abort();
 		}
@@ -386,7 +390,7 @@ public:
 		for (size_t c = 0; c < actions.size(); c++) {
 			if (actions[c]) {
 				if (actions[c]->GetRef()>2) {
-					printf("Residue action %d with refcount %d\n", actions[c]->actionID, actions[c]->GetRef());
+					print("Residue action %d with refcount %d\n", actions[c]->actionID, actions[c]->GetRef());
 				}
 				actions[c]->Release();
 				actions[c] = NULL;
@@ -503,6 +507,7 @@ typedef int (* IDSFunction)(Actor *, int parameter);
 
 #define TF_NONE 	0
 #define TF_CONDITION    1 //this isn't a trigger, just a condition (0x4000)
+#define TF_SAVED        2 //trigger is in svtriobj.ids
 #define TF_MERGESTRINGS 8 //same value as actions' mergestring
 
 struct TriggerLink {
@@ -786,6 +791,7 @@ public: //Script Functions
 	static int ItemIsIdentified(Scriptable* Sender, Trigger* parameters);
 	static int Joins(Scriptable* Sender, Trigger* parameters);
 	static int Kit(Scriptable* Sender, Trigger* parameters);
+	static int Killed(Scriptable* Sender, Trigger* parameters);
 	static int KnowSpell(Scriptable* Sender, Trigger* parameters);
 	static int LastMarkedObject_Trigger(Scriptable* Sender, Trigger* parameters);
 	static int LastPersonTalkedTo(Scriptable* Sender, Trigger* parameters);
@@ -1037,6 +1043,7 @@ public:
 	static void DayNight(Scriptable *Sender, Action* parameters);
 	static void Deactivate(Scriptable* Sender, Action* parameters);
 	static void Debug(Scriptable* Sender, Action* parameters);
+	static void DemoEnd(Scriptable* Sender, Action* parameters);
 	static void DestroyAllDestructableEquipment(Scriptable* Sender,
 		Action* parameters);
 	static void DestroyAllEquipment(Scriptable* Sender, Action* parameters);
@@ -1399,6 +1406,7 @@ public:
 	static void TransformPartyItem(Scriptable* Sender, Action* parameters);
 	static void TransformPartyItemAll(Scriptable* Sender, Action* parameters);
 	static void TriggerActivation(Scriptable* Sender, Action* parameters);
+	static void TriggerWalkTo(Scriptable* Sender, Action* parameters);
 	static void Turn(Scriptable* Sender, Action* parameters);
 	static void TurnAMT(Scriptable* Sender, Action* parameters);
 	static void UndoExplore(Scriptable *Sender, Action *parameters);
