@@ -28,15 +28,16 @@
 #include "exports.h"
 #include "ie_types.h"
 
-#include "Animation.h"
 #include "Audio.h"
-#include "CharAnimations.h"
 #include "EffectQueue.h"
-#include "ScriptedAnimation.h"
+#include "Palette.h"
 
 #include <cstring>
 #include <vector>
 
+class Animation;
+class CharAnimations;
+class DataFileMgr;
 class Map;
 class ScriptedAnimation;
 struct PolymorphCache;
@@ -134,9 +135,13 @@ struct PolymorphCache;
 #define VCONST_COUNT 100
 
 //interact types
+#define I_NONE       0
 #define I_INSULT     1
 #define I_COMPLIMENT 2
 #define I_SPECIAL    3
+#define I_INSULT_RESP 4
+#define I_COMPL_RESP 5
+#define I_DIALOG     6
 
 // 3 for blur, 8 for mirror images
 #define EXTRA_ACTORCOVERS 11
@@ -275,25 +280,6 @@ public:
 	ieDword ModalState;
 	int PathTries; //the # of previous tries to pick up a new walkpath
 public:
-	#define LastTarget LastDisarmFailed
-	//ieDword LastTarget; use lastdisarmfailed
-	#define LastAttacker LastDisarmed
-	//ieDword LastAttacker; use lastdisarmed
-	#define LastHitter LastEntered
-	//ieDword LastHitter; use lastentered
-	#define LastSummoner LastTrigger
-	//ieDword LastSummoner; use lasttrigger
-	#define LastTalkedTo LastUnlocked
-	//ieDword LastTalkedTo; use lastunlocked
-	ieDword LastProtected;
-	ieDword LastFollowed;
-	ieDword LastCommander;
-	ieDword LastHelp;
-	ieDword LastSeen;
-	ieDword LastMarked;  //no idea if non-actors could mark objects
-	int LastMarkedSpell; //a spell number to cast
-	ieDword LastHeard;
-	ieDword HotKey;
 	ieDword LastExit;    //the global ID of the exit to be used
 	char ShieldRef[2];
 	char HelmetRef[2];
@@ -303,11 +289,7 @@ public:
 	bool GotLUFeedback;
 	int WMLevelMod;
 
-	int LastCommand;   //lastcommander
-	int LastShout;     //lastheard
-	int LastDamage;    //lasthitter
-	int LastDamageType;//lasthitter
-	ieDword LastTurner;
+	int LastDamageType;
 	Point FollowOffset;//follow lastfollowed at this offset
 	Point HomeLocation;//spawnpoint, return here after rest
 
@@ -322,6 +304,8 @@ public:
 	ieDword modalTime;           //last time the modal effect used
 	char modalSpellLingering;    //the count of rounds for which the modal spell will be reapplied after the state ends
 	ieDword panicMode;           //runaway, berserk or randomwalk
+	ieDword nextComment;         //do something random (area comment, interaction)
+	ieDword nextBored;           //do something when bored
 	ieDword lastInit;
 	bool no_more_steps;
 	int speed;
@@ -369,6 +353,8 @@ private:
 	ieDword GetClassLevel (const ieDword id) const;
 	/* Returns true if the dual class is backwards */
 	bool IsDualSwap() const;
+	/* returns the weapon proficiency stat of the actor */
+	int GetProficiency(int proftype) const;
 	/** Re/Inits the Modified vector for PCs/NPCs */
 	void RefreshPCStats();
 	bool ShouldHibernate();
@@ -478,7 +464,13 @@ public:
 	/* receives undead turning message */
 	void Turn(Scriptable *cleric, ieDword turnlevel);
 	/* call this on gui selects */
-	void SelectActor();
+	void SelectActor() const;
+	/* call this when adding actions via gui */
+	void CommandActor() const;
+	/** handle panic and other involuntary actions that mess with scripting */
+	bool OverrideActions();
+	/** handle idle actions, that shouldn't mess with scripting */
+	void IdleActions(bool nonidle);
 	/* sets the actor in panic (turn/morale break) */
 	void Panic(Scriptable *attacker, int panicmode);
 	/* sets a multi class flag (actually this is a lot of else too) */
@@ -488,9 +480,9 @@ public:
 	/* returns a remapped verbal constant strref */
 	ieStrRef GetVerbalConstant(int index) const;
 	/* displaying a random verbal constant */
-	void VerbalConstant(int start, int count);
+	void VerbalConstant(int start, int count) const;
 	/* inlined dialogue response */
-	void Response(int type);
+	void Response(int type) const;
 	/* called when someone died in the party */
 	void ReactToDeath(const char *deadname);
 	/* called when someone talks to Actor */
@@ -603,6 +595,12 @@ public:
 	void ResolveStringConstant(ieResRef sound, unsigned int index) const;
 	void GetSoundFromINI(ieResRef Sound, unsigned int index) const;
 	void GetSoundFrom2DA(ieResRef Sound, unsigned int index) const;
+	/* generate area specific oneliner */
+	void GetAreaComment(int areaflag) const;
+	/* handle oneliner interaction, -1: unsuccessful (may comment area), 0: dialog banter, 1: oneliner */
+	int HandleInteract(Actor *target);
+	/* generate party banter, return true if successful */
+	bool GetPartyComment();
 	/* sets the quick slots */
 	void SetActionButtonRow(ActionButtonRow &ar);
 	/* updates the quick slots */
