@@ -25,10 +25,14 @@
 
 #include "win32def.h"
 
+#include "Animation.h"
+#include "AnimationFactory.h"
 #include "Audio.h"
 #include "Game.h"
 #include "GameData.h"
 #include "Interface.h"
+#include "Map.h"
+#include "Sprite2D.h"
 #include "Video.h"
 
 #define ILLEGAL 0         //
@@ -226,7 +230,7 @@ void ScriptedAnimation::LoadAnimationFactory(AnimationFactory *af, int gettwin)
 }
 
 /* Creating animation from VVC */
-ScriptedAnimation::ScriptedAnimation(DataStream* stream, bool autoFree)
+ScriptedAnimation::ScriptedAnimation(DataStream* stream)
 {
 	Init();
 	if (!stream) {
@@ -237,9 +241,8 @@ ScriptedAnimation::ScriptedAnimation(DataStream* stream, bool autoFree)
 
 	stream->Read( Signature, 8);
 	if (strncmp( Signature, "VVC V1.0", 8 ) != 0) {
-		printf( "Not a valid VVC File\n" );
-		if (autoFree)
-			delete( stream );
+		print( "Not a valid VVC File\n" );
+		delete stream;
 		return;
 	}
 	ieResRef Anim1ResRef;
@@ -383,9 +386,7 @@ ScriptedAnimation::ScriptedAnimation(DataStream* stream, bool autoFree)
 
 	SetPhase(P_ONSET);
 
-	if (autoFree) {
-		delete( stream );
-	}
+	delete stream;
 }
 
 ScriptedAnimation::~ScriptedAnimation(void)
