@@ -164,7 +164,7 @@ bool Spellbook::HaveSpell(int spellid, ieDword flags)
 				if (atoi(ms->SpellResRef+4)==spellid) {
 					if (flags&HS_DEPLETE) {
 						if (DepleteSpell(ms) && (sorcerer & (1<<type) ) ) {
-							DepleteLevel (sm);
+							DepleteLevel (sm, ms->SpellResRef);
 						}
 					}
 					return true;
@@ -231,7 +231,7 @@ bool Spellbook::HaveSpell(const char *resref, ieDword flags)
 					}
 					if (flags&HS_DEPLETE) {
 						if (DepleteSpell(ms) && (sorcerer & (1<<i) ) ) {
-							DepleteLevel (sm);
+							DepleteLevel (sm, ms->SpellResRef);
 						}
 					}
 					return true;
@@ -285,7 +285,7 @@ unsigned int Spellbook::GetTotalMemorizedSpellsCount() const
 	for (int type = 0; type < NUM_BOOK_TYPES; type++) {
 		unsigned int level = GetSpellLevelCount(type);
 		while(level--) {
-			total += GetMemorizedSpellsCount(type, level);
+			total += GetMemorizedSpellsCount(type, level, false);
 		}
 	}
 	return total;
@@ -470,25 +470,69 @@ CREKnownSpell* Spellbook::GetKnownSpell(int type, unsigned int level, unsigned i
 	return spells[type][level]->known_spells[index];
 }
 
-unsigned int Spellbook::GetMemorizedSpellsCount(int type) const
+unsigned int Spellbook::GetMemorizedSpellsCount(int type, bool real) const
 {
 	unsigned int count = 0;
 	size_t i=GetSpellLevelCount(type);
 	while(i--) {
-		count += (unsigned int) spells[type][i]->memorized_spells.size();
+		if (real) {
+			int j = spells[type][i]->memorized_spells.size();
+			while(j--) {
+				if (spells[type][i]->memorized_spells[j]->Flags) count++;
+			}
+		} else {
+			count += (unsigned int) spells[type][i]->memorized_spells.size();
+		}
 	}
 	return count;
 }
 
-unsigned int Spellbook::GetMemorizedSpellsCount(int type, unsigned int level) const
+unsigned int Spellbook::GetMemorizedSpellsCount(int type, unsigned int level, bool real) const
 {
 	if (type >= NUM_BOOK_TYPES)
 		return 0;
 	if (level >= GetSpellLevelCount(type))
 		return 0;
+	if (real) {
+		unsigned int count = 0;
+		int j = spells[type][level]->memorized_spells.size();
+		while(j--) {
+			if (spells[type][level]->memorized_spells[j]->Flags) count++;
+		}
+		return count;
+	}
 	return (unsigned int) spells[type][level]->memorized_spells.size();
 }
 
+unsigned int Spellbook::GetMemorizedSpellsCount(const ieResRef name, int type, bool real) const
+{
+	if (type >= NUM_BOOK_TYPES)
+		return 0;
+	int t;
+	if (type<0) {
+		t = NUM_BOOK_TYPES-1;
+	} else {
+		t = type;
+	}
+
+	int j = 0;
+	while(t>=0) {
+		unsigned int level = GetSpellLevelCount(t);
+		while(level--) {
+			int i = spells[t][level]->memorized_spells.size();
+			while(i--) {
+				CREMemorizedSpell *cms = spells[t][level]->memorized_spells[i];
+
+				if (strnicmp(cms->SpellResRef, name, sizeof(ieResRef) ) ) continue;
+				if (!real || cms->Flags) j++;
+			}
+		}
+		if (type>=0) break;
+		t--;
+	}
+	return j;
+}
+
 CREMemorizedSpell* Spellbook::GetMemorizedSpell(int type, unsigned int level, unsigned int index) const
 {
 	if (type >= NUM_BOOK_TYPES || level >= GetSpellLevelCount(type) || index >= spells[type][level]->memorized_spells.size())
@@ -755,7 +799,7 @@ bool Spellbook::DepleteSpell(int type)
 		for (unsigned int k = 0; k < sm->memorized_spells.size(); k++) {
 			if (DepleteSpell( sm->memorized_spells[k] )) {
 				if (sorcerer & (1<<type) ) {
-					DepleteLevel (sm);
+					DepleteLevel (sm, sm->memorized_spells[k]->SpellResRef);
 				}
 				return true;
 			}
@@ -764,19 +808,23 @@ bool Spellbook::DepleteSpell(int type)
 	return false;
 }
 
-void Spellbook::DepleteLevel(CRESpellMemorization* sm)
+void Spellbook::DepleteLevel(CRESpellMemorization* sm, const ieResRef except)
 {
 	size_t cnt = sm->memorized_spells.size();
 	ieResRef last={""};
+
 	for (size_t i = 0; i < cnt && cnt>0; i++) {
 		CREMemorizedSpell *cms = sm->memorized_spells[i];
 		//sorcerer spells are created in orderly manner
-		if (strncmp(last,cms->SpellResRef,8) ) {
+		if (cms->Flags && strncmp(last,cms->SpellResRef,8) && strncmp(except,cms->SpellResRef,8)) {
 			memcpy(last, cms->SpellResRef, sizeof(ieResRef) );
+			cms->Flags=0;
+/*
 			delete cms;
 			sm->memorized_spells.erase(sm->memorized_spells.begin()+i);
 			i--;
 			cnt--;
+*/
 		}
 	}
 }
@@ -799,7 +847,7 @@ bool Spellbook::DepleteSpell(int type, unsigned int page, unsigned int slot)
 	CREMemorizedSpell* cms = sm->memorized_spells[slot];
 	ret = DepleteSpell(cms);
 	if (ret && (sorcerer & (1<<type) ) ) {
-		DepleteLevel (sm);
+		DepleteLevel (sm, cms->SpellResRef);
 	}
 
 	return ret;
@@ -878,6 +926,20 @@ unsigned int Spellbook::GetSpellInfoSize(int type)
 	return count;
 }
 
+bool Spellbook::FindSpellInfo(SpellExtHeader *array, ieResRef spellname)
+{
+	memset(array, 0, sizeof(SpellExtHeader) );
+	if (spellinfo.size() == 0) {
+		GenerateSpellInfo();
+	}
+	for (unsigned int i = 0; i<spellinfo.size(); i++) {
+		if (strnicmp(spellinfo[i]->spellname, spellname, sizeof(ieResRef) ) ) continue;
+		memcpy(array, spellinfo[i], sizeof(SpellExtHeader));
+		return true;
+	}
+	return false;
+}
+
 SpellExtHeader *Spellbook::FindSpellInfo(unsigned int level, unsigned int type, const ieResRef spellname)
 {
 	size_t i = spellinfo.size();
@@ -908,16 +970,16 @@ void Spellbook::AddSpellInfo(unsigned int sm_level, unsigned int sm_type, const
 
 	seh = new SpellExtHeader;
 	spellinfo.push_back( seh );
- 
+
 	memcpy(seh->spellname, spellname, sizeof(ieResRef) );
 	int ehc;
-	
+
 	for (ehc = 0; ehc < spl->ExtHeaderCount-1; ehc++) {
 		if (level<spl->ext_headers[ehc+1].RequiredLevel) {
 			break;
 		}
 	}
-	
+
 	SPLExtHeader *ext_header = spl->ext_headers+ehc;
 	seh->headerindex = ehc;
 	seh->level = sm_level;
@@ -989,27 +1051,27 @@ void Spellbook::dump()
 {
 	unsigned int k;
 
-	printf( "SPELLBOOK:\n" );
+	print( "SPELLBOOK:\n" );
 	for (int i = 0; i < NUM_BOOK_TYPES; i++) {
 		for (unsigned int j = 0; j < spells[i].size(); j++) {
 			CRESpellMemorization* sm = spells[i][j];
 
 			if (sm->known_spells.size())
-				printf( " Known spells:\n" );
+				print( " Known spells:\n" );
 			for (k = 0; k < sm->known_spells.size(); k++) {
 				CREKnownSpell* spl = sm->known_spells[k];
 				if (!spl) continue;
 
-				printf ( " %2d: %8s L: %d T: %d\n", k, spl->SpellResRef, spl->Level, spl->Type );
+				print ( " %2d: %8s L: %d T: %d\n", k, spl->SpellResRef, spl->Level, spl->Type );
 			}
 
 			if (sm->memorized_spells.size())
-				printf( " Memorized spells:\n" );
+				print( " Memorized spells:\n" );
 			for (k = 0; k < sm->memorized_spells.size (); k++) {
 				CREMemorizedSpell* spl = sm->memorized_spells[k];
 				if (!spl) continue;
 
-				printf ( " %2u: %8s %x\n", k, spl->SpellResRef, spl->Flags );
+				print ( " %2u: %8s %x\n", k, spl->SpellResRef, spl->Flags );
 			}
 		}
 	}
