@@ -22,6 +22,10 @@
 #define CREIMPORTER_H
 
 #include "ActorMgr.h"
+#include "Spellbook.h"
+
+class CREItem;
+struct Effect;
 
 #define IE_CRE_GEMRB            0
 #define IE_CRE_V1_0		10  //bg1
@@ -33,7 +37,6 @@
 class CREImporter : public ActorMgr {
 private:
 	DataStream* str;
-	bool autoFree;
 	unsigned char CREVersion;
 	ieDword KnownSpellsOffset;
 	ieDword KnownSpellsCount;
@@ -61,7 +64,7 @@ private:
 public:
 	CREImporter(void);
 	~CREImporter(void);
-	bool Open(DataStream* stream, bool autoFree = true);
+	bool Open(DataStream* stream);
 	Actor* GetActor(unsigned char is_in_party);
 
 	//returns saved size, updates internal offsets before save
