@@ -29,8 +29,9 @@ class Resource;
 class ResourceDesc;
 
 class DirectoryImporter : public ResourceSource {
-private:
+protected:
 	char path[_MAX_PATH];
+
 public:
 	DirectoryImporter(void);
 	~DirectoryImporter(void);
@@ -43,4 +44,23 @@ public:
 	DataStream* GetResource(const char* resname, const ResourceDesc &type);
 };
 
+class CachedDirectoryImporter : public DirectoryImporter {
+protected:
+	std::map<std::string, std::string> cache;
+
+public:
+	CachedDirectoryImporter();
+	~CachedDirectoryImporter();
+
+	bool Open(const char *dir, const char *desc);
+	void Refresh();
+	/** predicts the availability of a resource */
+	bool HasResource(const char* resname, SClass_ID type);
+	bool HasResource(const char* resname, const ResourceDesc &type);
+	/** returns resource */
+	DataStream* GetResource(const char* resname, SClass_ID type);
+	DataStream* GetResource(const char* resname, const ResourceDesc &type);
+};
+
+
 #endif
