@@ -24,7 +24,10 @@
 #include "EffectQueue.h"
 #include "Game.h"
 #include "GameData.h"
+#include "GlobalTimer.h"
 #include "Interface.h"
+#include "Map.h"
+#include "TableMgr.h"
 #include "TileMap.h"
 #include "Video.h" //for tints
 #include "Scriptable/Actor.h"
@@ -94,7 +97,7 @@ void RegisterTormentOpcodes()
 //retreat_from (works only in PST) - forces target to run away/walk away from Owner
 int fx_retreat_from (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_retreat_from (%2d): Mod: %d, Type: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
+	if (0) print( "fx_retreat_from (%2d): Mod: %d, Type: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
 
 	if (!Owner) {
 		return FX_NOT_APPLIED;
@@ -125,7 +128,7 @@ int fx_retreat_from (Scriptable* Owner, Actor* target, Effect* fx)
 //0xba fx_set_status
 int fx_set_status (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_set_status (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_set_status (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	if (fx->Parameter1) {
 		if (fx->TimingMode==FX_DURATION_INSTANT_PERMANENT) {
 			BASE_STATE_SET (fx->Parameter2);
@@ -149,7 +152,7 @@ int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx)
 {
 	bool playonce;
 
-	if (0) printf( "fx_play_bam_blended (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_play_bam_blended (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	if (!Owner)
 		Owner = target;
 	if (!Owner)
@@ -226,7 +229,7 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 	bool playonce;
 	bool doublehint;
 
-	if (0) printf( "fx_play_bam_not_blended (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_play_bam_not_blended (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	if (!Owner)
 		Owner = target;
 	if (!Owner)
@@ -332,7 +335,7 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 //0xc0 fx_transfer_hp
 int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_transfer_hp (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_transfer_hp (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	if (Owner->Type!=ST_ACTOR) {
 		return FX_NOT_APPLIED;
 	}
@@ -383,7 +386,7 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 //0xc2 fx_flash_screen
 int fx_flash_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) printf( "fx_flash_screen (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_flash_screen (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	core->GetVideoDriver()->SetFadeColor(((unsigned char *) &fx->Parameter1)[0],((unsigned char *) &fx->Parameter1)[1],((unsigned char *) &fx->Parameter1)[2]);
 	//this needs to be at least 2 for any effect
 	core->timer->SetFadeFromColor(2);
@@ -394,7 +397,7 @@ int fx_flash_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 //FIXME: implement bit4 which would mean duration
 int fx_tint_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) printf( "fx_tint_screen (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_tint_screen (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	int fromTime = fx->DiceSides;
 	int toTime = fx->DiceSides;
 	switch(fx->Parameter2&6) {
@@ -410,7 +413,7 @@ int fx_tint_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 //it is a mystery, why they needed to make this effect
 int fx_special_effect (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_special_effect (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_special_effect (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	//param2 determines the effect's behaviour
 	//0 - adder's kiss projectile (0xcd)
 	//  adds play bam and damage opcodes to the projectile
@@ -439,7 +442,7 @@ int fx_special_effect (Scriptable* Owner, Actor* target, Effect* fx)
 //it plays multiple vvc's with a given delay and duration
 int fx_multiple_vvc (Scriptable* Owner, Actor* /*target*/, Effect* fx)
 {
-	if (0) printf( "fx_multiple_vvc (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_multiple_vvc (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 
 	Map *area = Owner->GetCurrentArea();
 	if (!area)
@@ -474,7 +477,7 @@ int fx_multiple_vvc (Scriptable* Owner, Actor* /*target*/, Effect* fx)
 //GemRB specific, to support BMP area background changes (desert hell projectile)
 int fx_change_background (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) printf( "fx_change_background (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_change_background (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	Map *map = core->GetGame()->GetCurrentArea();
 	if (map) {
 		map->SetBackground(fx->Resource, fx->Duration);
@@ -486,7 +489,7 @@ int fx_change_background (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 //0xc9 fx_overlay
 int fx_overlay (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_overlay (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_overlay (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	target->AddAnimation(fx->Resource,-1,0,true);
 	//special effects based on fx_param2
 	return FX_NOT_APPLIED;
@@ -499,7 +502,7 @@ int fx_overlay (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //but i would rather use the IWD2 method
 int fx_bless (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_curse (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
+	if (0) print( "fx_curse (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
 	//this bit is the same as the invisibility bit in other games
 	//it should be considered what if we replace the pst invis bit
 	//with this one (losing binary compatibility, gaining easier
@@ -522,7 +525,7 @@ int fx_bless (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //0xcb fx_curse
 int fx_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_curse (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
+	if (0) print( "fx_curse (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
 	//this bit is the same as the invisibility bit in other games
 	//it should be considered what if we replace the pst invis bit
 	//with this one (losing binary compatibility, gaining easier
@@ -545,7 +548,7 @@ static EffectRef fx_bless_ref = { "Bless", -1 };
 
 int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_prayer (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
+	if (0) print( "fx_prayer (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
 	int ea = target->GetStat(IE_EA);
 	int type;
 	if (ea>EA_EVILCUTOFF) type = 1;
@@ -576,7 +579,7 @@ int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 //0xcd fx_move_view
 int fx_move_view (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) printf( "fx_move_view (%2d): Speed: %d\n", fx->Opcode, fx->Parameter1 );
+	if (0) print( "fx_move_view (%2d): Speed: %d\n", fx->Opcode, fx->Parameter1 );
 	Map *map = core->GetGame()->GetCurrentArea();
 	if (map) {
 		core->timer->SetMoveViewPort( fx->PosX, fx->PosY, fx->Parameter1, true);
@@ -587,7 +590,7 @@ int fx_move_view (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 //0xce fx_embalm
 int fx_embalm (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_embalm (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_embalm (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	if (STATE_GET (STATE_EMBALM) ) //embalm is non cumulative
 		return FX_NOT_APPLIED;
 	STATE_SET( STATE_EMBALM );
@@ -610,7 +613,7 @@ int fx_embalm (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //0xcf fx_stop_all_action
 int fx_stop_all_action (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) printf( "fx_stop_all_action (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if (0) print( "fx_stop_all_action (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
 	if (fx->Parameter2) {
 		core->GetGame()->TimeStop(NULL, 0xffffffff);
 	} else {
@@ -625,7 +628,7 @@ int fx_iron_fist (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
 	ieDword p1,p2;
 
-	if (0) printf( "fx_iron_fist (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
+	if (0) print( "fx_iron_fist (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
 	switch (fx->Parameter2)
 	{
 	case 0: p1 = 3; p2 = 6; break;
@@ -641,7 +644,7 @@ int fx_iron_fist (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //0xd1 fx_hostile_image
 int fx_hostile_image (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) printf( "fx_hostile_image (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
+	if (0) print( "fx_hostile_image (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
 	return FX_NOT_APPLIED;
 }
 
@@ -650,7 +653,7 @@ static EffectRef fx_single_color_pulse_ref = { "Color:BriefRGB", -1 };
 
 int fx_detect_evil (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_detect_evil (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
+	if (0) print( "fx_detect_evil (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
 	ieDword type = fx->Parameter2;
 	//default is alignment/evil/speed 30/range 10
 	if (!type) type = 0x08031e0a;
@@ -677,7 +680,7 @@ int fx_detect_evil (Scriptable* Owner, Actor* target, Effect* fx)
 //0xd3 fx_jumble_curse
 int fx_jumble_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) printf( "fx_jumble_curse (%2d)\n", fx->Opcode );
+	if (0) print( "fx_jumble_curse (%2d)\n", fx->Opcode );
 
 	if (STATE_GET( STATE_DEAD) ) {
 		return FX_NOT_APPLIED;
@@ -709,7 +712,7 @@ int fx_jumble_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //In GemRB it is used in a custom spell
 int fx_speak_with_dead (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) printf("fx_speak_with_dead (%2d)\n", fx->Opcode );
+	if (0) print("fx_speak_with_dead (%2d)\n", fx->Opcode );
 	if (!STATE_GET( STATE_DEAD) ) {
 		return FX_NOT_APPLIED;
 	}
