@@ -28,13 +28,31 @@
 #include "GameData.h"
 #include "Interface.h"
 #include "Palette.h"
+#include "Polygon.h"
 #include "SpriteCover.h"
 #include "GUI/Console.h"
+#include "GUI/EventMgr.h"
+#include "GUI/Window.h"
 
 #include <cmath>
 #include <cassert>
 #include <cstdio>
 
+#ifdef TARGET_OS_IPHONE
+#include "SDL_uikitkeyboard.h"
+#endif
+#ifdef ANDROID
+#include "SDL_screenkeyboard.h"
+#endif
+
+#if SDL_VERSION_ATLEAST(1,3,0) 
+#define GEM_SetPalette(surface, flags, colors, fcolor, ncolors)\
+SDL_SetPaletteColors((surface)->format->palette, colors, fcolor, ncolors)
+#else
+#define GEM_SetPalette(surface, flags, colors, fcolor, ncolors)\
+SDL_SetPalette( surface, flags, colors, fcolor, ncolors )
+#endif
+
 SDLVideoDriver::SDLVideoDriver(void)
 {
 	CursorIndex = 0;
@@ -47,7 +65,7 @@ SDLVideoDriver::SDLVideoDriver(void)
 	xCorr = 0;
 	yCorr = 0;
 	lastTime = 0;
-	GetTime( lastMouseTime );
+	lastMouseTime = GetTickCount();
 	backBuf=NULL;
 	extra=NULL;
 	subtitlestrref = 0;
@@ -73,12 +91,12 @@ SDLVideoDriver::~SDLVideoDriver(void)
 
 int SDLVideoDriver::Init(void)
 {
-	//printf("[SDLVideoDriver]: Init...");
+	//print("[SDLVideoDriver]: Init...");
 	if (SDL_InitSubSystem( SDL_INIT_VIDEO ) == -1) {
-		//printf("[ERROR]\n");
+		//print("[ERROR]\n");
 		return GEM_ERROR;
 	}
-	//printf("[OK]\n");
+	//print("[OK]\n");
 	SDL_EnableUNICODE( 1 );
 	SDL_EnableKeyRepeat( 500, 50 );
 	SDL_ShowCursor( SDL_DISABLE );
@@ -97,6 +115,11 @@ int SDLVideoDriver::CreateDisplay(int w, int h, int b, bool fs)
 	ieDword flags = SDL_SWSURFACE;
 	if (fullscreen) {
 		flags |= SDL_FULLSCREEN;
+#if SDL_VERSION_ATLEAST(1,3,0)
+		//This is needed to remove the status bar on Android/iOS. 
+		//since we are in fullscreen this has no effect outside Android/iOS
+		flags |= SDL_WINDOW_BORDERLESS;
+#endif
 	}
 	printMessage( "SDLVideo", "SDL_SetVideoMode...", WHITE );
 	disp = SDL_SetVideoMode( w, h, bpp, flags );
@@ -118,7 +141,7 @@ int SDLVideoDriver::CreateDisplay(int w, int h, int b, bool fs)
 	Viewport.h = height;
 	printMessage( "SDLVideo", "Creating Main Surface...", WHITE );
 	SDL_Surface* tmp = SDL_CreateRGBSurface( SDL_SWSURFACE, width, height,
-						bpp, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff );
+						bpp, 0, 0, 0, 0 );
 	printStatus( "OK", LIGHT_GREEN );
 	printMessage( "SDLVideo", "Creating Back Buffer...", WHITE );
 	backBuf = SDL_DisplayFormat( tmp );
@@ -141,13 +164,10 @@ void SDLVideoDriver::SetDisplayTitle(char* title, char* icon)
 	SDL_WM_SetCaption( title, icon );
 }
 
-bool SDLVideoDriver::ToggleFullscreenMode(int set_reset)
+bool SDLVideoDriver::SetFullscreenMode(bool set)
 {
-	if (set_reset==-1) {
-		 set_reset=!fullscreen;
-	}
-	if (fullscreen != set_reset) {
-		fullscreen=set_reset;
+	if (fullscreen != set) {
+		fullscreen=set;
 		SDL_WM_ToggleFullScreen( disp );
 		//readjust mouse to original position
 		MoveMouse(CursorPos.x, CursorPos.y);
@@ -171,10 +191,10 @@ int SDLVideoDriver::SwapBuffers(void)
 {
 	int ret = GEM_OK;
 	unsigned long time;
-	GetTime( time );
-	if (( time - lastTime ) < 17) {
-		SDL_Delay( 17 - (time - lastTime) );
-		GetTime( time );
+	time = GetTickCount();
+	if (( time - lastTime ) < 33) {
+		SDL_Delay( 33 - (time - lastTime) );
+		time = GetTickCount();
 	}
 	lastTime = time;
 
@@ -214,7 +234,7 @@ int SDLVideoDriver::SwapBuffers(void)
 	unsigned int delay = core->TooltipDelay;
 	// The multiplication by 10 is there since the last, disabling slider position is the eleventh
 	if (!ConsolePopped && (delay<TOOLTIP_DELAY_FACTOR*10) ) {
-		GetTime( time );
+		time = GetTickCount();
 		/** Display tooltip if mouse is idle */
 		if (( time - lastMouseTime ) > delay) {
 			if (Evnt)
@@ -266,7 +286,7 @@ int SDLVideoDriver::PollEvents() {
 					break;
 				case SDLK_f:
 					if (modstate & GEM_MOD_CTRL) {
-						ToggleFullscreenMode(-1);
+						ToggleFullscreenMode();
 					}
 					break;
 				default:
@@ -308,12 +328,16 @@ int SDLVideoDriver::PollEvents() {
 				case SDLK_RIGHT:
 					key = GEM_RIGHT;
 					break;
+				case SDLK_DELETE:
+#ifndef TARGET_OS_IPHONE
+					//iOS currently doesnt have a backspace so we use delete. 
+					//This change should be future proof in the event apple changes the delete key to a backspace.
+					key = GEM_DELETE;
+					break;
+#endif
 				case SDLK_BACKSPACE:
 					key = GEM_BACKSP;
 					break;
-				case SDLK_DELETE:
-					key = GEM_DELETE;
-					break;
 				case SDLK_RETURN:
 				case SDLK_KP_ENTER:
 					key = GEM_RETURN;
@@ -391,11 +415,23 @@ int SDLVideoDriver::PollEvents() {
 					Evnt->OnSpecialKeyPress( GEM_MOUSEOUT );
 			}
 			break;
-
+#if SDL_VERSION_ATLEAST(1,3,0)
+		case SDL_MOUSEWHEEL://SDL 1.3+
+			int scrollX;
+			scrollX= event.wheel.x;
+			int scrollY;
+			scrollY= event.wheel.y;
+			Evnt->MouseWheelScroll( scrollX, scrollY );
+			break;
+		default:
+			//this is to catch unhandled SDL 1.3 events for development
+			printMessage( "SDLVideo", "Unrecognized SDL event type.\n", LIGHT_RED );
+			print("event type:%x\n", event.type);
+#endif
 		}
 	}
 
-	if (Evnt && !DisableMouse && lastevent && time>lastmousetime && SDL_GetMouseState(&x,&y)==SDL_BUTTON(1)) {
+	if (Evnt && !DisableMouse && lastevent && time>lastmousetime && SDL_GetMouseState(&x,&y)==SDL_BUTTON(SDL_BUTTON_LEFT)) {
 		lastmousetime=time+Evnt->GetRKDelay();
 		if (!ConsolePopped)
 			Evnt->MouseUp( x, y, 1 << ( 0 ), GetModState(SDL_GetModState()) );
@@ -417,6 +453,37 @@ bool SDLVideoDriver::ToggleGrabInput()
 	}
 }
 
+/*
+This method is intended for devices with no physical keyboard or with an optional soft keyboard (iOS/Android)
+*/
+void SDLVideoDriver::HideSoftKeyboard()
+{
+
+	if(core->UseSoftKeyboard){
+#ifdef TARGET_OS_IPHONE
+		SDL_iPhoneKeyboardHide(SDL_GetFocusWindow());
+#endif
+#ifdef ANDROID
+		SDL_ANDROID_SetScreenKeyboardShown(0);
+#endif
+	}
+}
+
+/*
+This method is intended for devices with no physical keyboard or with an optional soft keyboard (iOS/Android)
+*/
+void SDLVideoDriver::ShowSoftKeyboard()
+{
+	if(core->UseSoftKeyboard){
+#ifdef TARGET_OS_IPHONE
+		SDL_iPhoneKeyboardShow(SDL_GetFocusWindow());
+#endif
+#ifdef ANDROID
+		SDL_ANDROID_SetScreenKeyboardShown(1);
+#endif
+	}
+}
+
 void SDLVideoDriver::InitSpriteCover(SpriteCover* sc, int flags)
 {
 	int i;
@@ -536,7 +603,7 @@ Sprite2D* SDLVideoDriver::CreateSprite8(int w, int h, int bpp, void* pixels,
 	} else {
 		colorcount = 16;
 	}
-	SDL_SetPalette( ( SDL_Surface * ) p, SDL_LOGPAL, ( SDL_Color * ) palette, 0, colorcount );
+	GEM_SetPalette( ( SDL_Surface * ) p, SDL_LOGPAL, ( SDL_Color * ) palette, 0, colorcount );
 	spr->vptr = p;
 	spr->pixels = pixels;
 	if (cK) {
@@ -816,7 +883,7 @@ void SDLVideoDriver::BlitSpriteRegion(const Sprite2D* spr, const Region& size, i
 	}
 }
 
-void SDLVideoDriver::BlitTile(const Sprite2D* spr, const Sprite2D* mask, int x, int y, const Region* clip, bool trans)
+void SDLVideoDriver::BlitTile(const Sprite2D* spr, const Sprite2D* mask, int x, int y, const Region* clip, unsigned int flags)
 {
 	if (spr->BAM) {
 		printMessage( "SDLVideo", "Tile blit not supported for this sprite\n", LIGHT_RED );
@@ -859,14 +926,18 @@ void SDLVideoDriver::BlitTile(const Sprite2D* spr, const Sprite2D* mask, int x,
 	const SDL_Color* pal = (( SDL_Surface * ) spr->vptr)->format->palette->colors;
 
 	const Uint8* mask_data = 0;
-	Uint8 ck = 0;
+	Uint32 ck = 0;
 	if (mask) {
 		mask_data = (Uint8*) (( SDL_Surface * ) mask->vptr)->pixels;
+#if SDL_VERSION_ATLEAST(1,3,0)
+		SDL_GetColorKey(( SDL_Surface * ) mask->vptr, &ck);
+#else
 		ck = (( SDL_Surface * ) mask->vptr)->format->colorkey;
+#endif
 	}
 
 	bool tint = false;
-	Color tintcol = {0,0,0,0};
+	Color tintcol = {255,255,255,0};
 
 	if (core->GetGame()) {
 		const Color* totint = core->GetGame()->GetGlobalTint();
@@ -882,27 +953,58 @@ void SDLVideoDriver::BlitTile(const Sprite2D* spr, const Sprite2D* mask, int x,
 		else \
 			BlitTile_internal<Uint16>(backBuf, x, y, rx, ry, w, h, data, pal, mask_data, ck, T, B); \
 
+	if (flags & TILE_GREY) {
 
-	if (trans) {
-		TRBlender_HalfTrans B(backBuf->format);
+		if (flags & TILE_HALFTRANS) {
+			TRBlender_HalfTrans B(backBuf->format);
 
-		if (tint) {
-			TRTinter_Tint T(tintcol);
+			TRTinter_Grey T(tintcol);
 			DO_BLIT
 		} else {
-			TRTinter_NoTint T;
+			TRBlender_Opaque B(backBuf->format);
+
+			TRTinter_Grey T(tintcol);
 			DO_BLIT
 		}
+
+	} else if (flags & TILE_SEPIA) {
+
+		if (flags & TILE_HALFTRANS) {
+			TRBlender_HalfTrans B(backBuf->format);
+
+			TRTinter_Sepia T(tintcol);
+			DO_BLIT
+		} else {
+			TRBlender_Opaque B(backBuf->format);
+
+			TRTinter_Sepia T(tintcol);
+			DO_BLIT
+		}
+
 	} else {
-		TRBlender_Opaque B(backBuf->format);
 
-		if (tint) {
-			TRTinter_Tint T(tintcol);
-			DO_BLIT
+		if (flags & TILE_HALFTRANS) {
+			TRBlender_HalfTrans B(backBuf->format);
+
+			if (tint) {
+				TRTinter_Tint T(tintcol);
+				DO_BLIT
+			} else {
+				TRTinter_NoTint T;
+				DO_BLIT
+			}
 		} else {
-			TRTinter_NoTint T;
-			DO_BLIT
+			TRBlender_Opaque B(backBuf->format);
+
+			if (tint) {
+				TRTinter_Tint T(tintcol);
+				DO_BLIT
+			} else {
+				TRTinter_NoTint T;
+				DO_BLIT
+			}
 		}
+
 	}
 
 #undef DO_BLIT
@@ -1309,7 +1411,7 @@ void SDLVideoDriver::BlitGameSprite(const Sprite2D* spr, int x, int y,
 		// tinted
 		// covered
 
-//		printf("Unoptimized blit: %04X\n", flags);
+//		print("Unoptimized blit: %04X\n", flags);
 
 #define SPECIALPIXEL   int ia=0; if ((remflags & BLIT_HALFTRANS) || (p == 1 && (remflags & BLIT_TRANSSHADOW))) ia = 1; if (p == 1 && (remflags & BLIT_NOSHADOW)) { } else
 
@@ -1439,7 +1541,7 @@ void SDLVideoDriver::BlitGameSprite(const Sprite2D* spr, int x, int y,
 		// transshadow  (impossible with 32bpp)
 		// palettealpha (always set)
 
-//		printf("Unoptimized blit: %04X\n", flags);
+//		print("Unoptimized blit: %04X\n", flags);
 
 #define SPECIALPIXEL   int ia=0; if ((remflags & BLIT_HALFTRANS)) ia = 1; if (p == 1 && (remflags & BLIT_NOSHADOW)) { } else
 
@@ -1555,8 +1657,8 @@ void SDLVideoDriver::SetDragCursor(Sprite2D* drag)
 
 Sprite2D* SDLVideoDriver::GetScreenshot( Region r )
 {
-	int Width = r.w ? r.w : disp->w;
-	int Height = r.h ? r.h : disp->h;
+	unsigned int Width = r.w ? r.w : disp->w;
+	unsigned int Height = r.h ? r.h : disp->h;
 	SDL_Rect src = {r.x, r.y, r.w, r.h};
 
 
@@ -1564,7 +1666,8 @@ Sprite2D* SDLVideoDriver::GetScreenshot( Region r )
 				0xFF0000, 0x00FF00, 0x0000FF, 0x000000 );
 	SDL_BlitSurface( backBuf, (r.w && r.h) ? &src : NULL, surf, NULL);
 	void* pixels = malloc( Width * Height * 3 );
-	memcpy( pixels, surf->pixels, Width * Height * 3 );
+	for (unsigned int y = 0; y < Height; y++)
+		memcpy( (char*)pixels+(Width * y * 3), (char*)surf->pixels+(surf->pitch * y), Width * 3 );
 	//freeing up temporary surface as we copied its pixels
 	Sprite2D* screenshot = CreateSprite( Width, Height, 24, 0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000, pixels, false, 0 );
 	SDL_FreeSurface(surf);
@@ -1575,7 +1678,7 @@ Sprite2D* SDLVideoDriver::GetScreenshot( Region r )
 void SDLVideoDriver::SetPalette(void *data, Palette* pal)
 {
 	SDL_Surface* sur = ( SDL_Surface* ) data;
-	SDL_SetPalette( sur, SDL_LOGPAL, ( SDL_Color * ) pal->col, 0, 256 );
+	GEM_SetPalette(sur, SDL_LOGPAL, ( SDL_Color * ) pal->col, 0, 256);
 }
 
 void SDLVideoDriver::ConvertToVideoFormat(Sprite2D* sprite)
@@ -1615,7 +1718,7 @@ void SDLVideoDriver::DrawRect(const Region& rgn, const Color& color, bool fill,
 			c.r = color.r;
 			c.b = color.b;
 			c.g = color.g;
-			SDL_SetPalette( rectsurf, SDL_LOGPAL, &c, 0, 1 );
+			GEM_SetPalette( rectsurf, SDL_LOGPAL, &c, 0, 1 );
 			SDL_SetAlpha( rectsurf, SDL_SRCALPHA | SDL_RLEACCEL, color.a );
 			SDL_BlitSurface( rectsurf, NULL, backBuf, &drect );
 			SDL_FreeSurface( rectsurf );
@@ -1651,7 +1754,7 @@ void SDLVideoDriver::DrawRectSprite(const Region& rgn, const Color& color, const
 		c.r = color.r;
 		c.b = color.b;
 		c.g = color.g;
-		SDL_SetPalette( rectsurf, SDL_LOGPAL, &c, 0, 1 );
+		GEM_SetPalette( rectsurf, SDL_LOGPAL, &c, 0, 1 );
 		SDL_SetAlpha( rectsurf, SDL_SRCALPHA | SDL_RLEACCEL, color.a );
 		SDL_BlitSurface( rectsurf, NULL, surf, &drect );
 		SDL_FreeSurface( rectsurf );
@@ -1696,7 +1799,7 @@ inline void WritePixel(const long val, unsigned char *pixels, int BytesPerPixel)
 
 void SDLVideoDriver::SetPixel(short x, short y, const Color& color, bool clipped)
 {
-	//printf("x: %d; y: %d; XC: %d; YC: %d, VX: %d, VY: %d, VW: %d, VH: %d\n", x, y, xCorr, yCorr, Viewport.x, Viewport.y, Viewport.w, Viewport.h);
+	//print("x: %d; y: %d; XC: %d; YC: %d, VX: %d, VY: %d, VW: %d, VH: %d\n", x, y, xCorr, yCorr, Viewport.x, Viewport.y, Viewport.w, Viewport.h);
 	if (clipped) {
 		x += xCorr;
 		y += yCorr;
@@ -2333,7 +2436,7 @@ void SDLVideoDriver::GetMousePos(int &x, int &y)
 
 void SDLVideoDriver::MouseMovement(int x, int y)
 {
-	GetTime( lastMouseTime );
+	lastMouseTime = GetTickCount();
 	if (DisableMouse&MOUSE_DISABLED)
 		return;
 	CursorPos.x = x; // - mouseAdjustX[CursorIndex];
@@ -2414,8 +2517,7 @@ void SDLVideoDriver::showFrame(unsigned char* buf, unsigned int bufw,
 		sprite = SDL_CreateRGBSurfaceFrom( buf, bufw, bufh, 16, 2 * bufw,
 					0x7C00, 0x03E0, 0x001F, 0 );
 	} else {
-		sprite = SDL_CreateRGBSurfaceFrom( buf, bufw, bufh, 8, bufw, 0x7C00,
-					0x03E0, 0x001F, 0 );
+		sprite = SDL_CreateRGBSurfaceFrom( buf, bufw, bufh, 8, bufw, 0, 0, 0, 0 );
 
 		for (i = 0; i < 256; i++) {
 			sprite->format->palette->colors[i].r = ( *pal++ ) << 2;
@@ -2494,7 +2596,7 @@ int SDLVideoDriver::PollMovieEvents()
 					case SDLK_q:
 						return 1;
 					case SDLK_f:
-						ToggleFullscreenMode(-1);
+						ToggleFullscreenMode();
 						break;
 					default:
 						break;
